Package: MFSIS
Type: Package
Title: Model-Free Sure Independent Screening Procedures
Version: 0.3.1
Date: 2026-05-12
Authors@R: c(
              person("Xuewei", "Cheng", role = c("aut","cre"), email = "xwcheng@hunnu.edu.cn"),
              person("Hong", "Wang", role = c("aut"), email = "wh@csu.edu.cn"),
              person("Liping", "Zhu", role = c("aut"), email = "zhu.liping@ruc.edu.cn"),
              person("Wei", "Zhong", role = c("aut"), email = "wzhong@xmu.edu.cn"),
              person("Hanpu", "Zhou", role = c("aut"), email = "zhouhanpu@csu.edu.cn")
              )
Author: Xuewei Cheng [aut, cre],
  Hong Wang [aut],
  Liping Zhu [aut],
  Wei Zhong [aut],
  Hanpu Zhou [aut]
Maintainer: Xuewei Cheng <xwcheng@hunnu.edu.cn>
Description: An implementation of popular screening methods that are commonly 
  employed in ultra-high and high dimensional data. Through this publicly 
  available package, we provide a unified framework to carry out model-free 
  screening procedures including 
  SIS (Fan and Lv (2008) <doi:10.1111/j.1467-9868.2008.00674.x>), 
  SIRS(Zhu et al. (2011)<doi:10.1198/jasa.2011.tm10563>), 
  DC-SIS (Li et al. (2012) <doi:10.1080/01621459.2012.695654>), 
  MDC-SIS(Shao and Zhang (2014) <doi:10.1080/01621459.2014.887012>), 
  Bcor-SIS (Pan et al. (2019) <doi:10.1080/01621459.2018.1462709>), 
  PC-Screen (Liu et al. (2020) <doi:10.1080/01621459.2020.1783274>), 
  WLS (Zhong et al.(2021) <doi:10.1080/01621459.2021.1918554>), 
  Kfilter (Mai and Zou (2015) <doi:10.1214/14-AOS1303>), 
  MVSIS (Cui et al. (2015) <doi:10.1080/01621459.2014.920256>), 
  PSIS (Pan et al. (2016) <doi:10.1080/01621459.2014.998760>), 
  CAS (Xie et al. (2020) <doi:10.1080/01621459.2019.1573734>), 
  CI-SIS (Cheng and Wang. (2023) <doi:10.1016/j.cmpb.2022.107269>), 
  CSIS (Cheng et al. (2024) <doi:10.1007/s00180-023-01399-5>) and Log-rank SIS.
License: GPL (>= 2)
SystemRequirements: Python (>= 3.8.0)
Encoding: UTF-8
Imports: survival, MASS, Ball, reticulate, stats, crayon, cli, dr,
        foreach, parallel, doParallel, fs
Suggests: knitr, utils, pkgdown
Language: en-US
NeedsCompilation: no
Repository: CRAN
RoxygenNote: 7.2.3
Packaged: 2026-05-13 17:54:22 UTC; cxw
Date/Publication: 2026-05-13 18:50:02 UTC
Built: R 4.5.2; ; 2026-05-13 19:54:26 UTC; unix

Package: MFSIS
Type: Package
Title: Model-Free Sure Independent Screening Procedures
Version: 0.3.1
Date: 2026-05-12
Authors@R: c(
              person("Xuewei", "Cheng", role = c("aut","cre"), email = "xwcheng@hunnu.edu.cn"),
              person("Hong", "Wang", role = c("aut"), email = "wh@csu.edu.cn"),
              person("Liping", "Zhu", role = c("aut"), email = "zhu.liping@ruc.edu.cn"),
              person("Wei", "Zhong", role = c("aut"), email = "wzhong@xmu.edu.cn"),
              person("Hanpu", "Zhou", role = c("aut"), email = "zhouhanpu@csu.edu.cn")
              )
Author: Xuewei Cheng [aut, cre],
  Hong Wang [aut],
  Liping Zhu [aut],
  Wei Zhong [aut],
  Hanpu Zhou [aut]
Maintainer: Xuewei Cheng <xwcheng@hunnu.edu.cn>
Description: An implementation of popular screening methods that are commonly 
  employed in ultra-high and high dimensional data. Through this publicly 
  available package, we provide a unified framework to carry out model-free 
  screening procedures including 
  SIS (Fan and Lv (2008) <doi:10.1111/j.1467-9868.2008.00674.x>), 
  SIRS(Zhu et al. (2011)<doi:10.1198/jasa.2011.tm10563>), 
  DC-SIS (Li et al. (2012) <doi:10.1080/01621459.2012.695654>), 
  MDC-SIS(Shao and Zhang (2014) <doi:10.1080/01621459.2014.887012>), 
  Bcor-SIS (Pan et al. (2019) <doi:10.1080/01621459.2018.1462709>), 
  PC-Screen (Liu et al. (2020) <doi:10.1080/01621459.2020.1783274>), 
  WLS (Zhong et al.(2021) <doi:10.1080/01621459.2021.1918554>), 
  Kfilter (Mai and Zou (2015) <doi:10.1214/14-AOS1303>), 
  MVSIS (Cui et al. (2015) <doi:10.1080/01621459.2014.920256>), 
  PSIS (Pan et al. (2016) <doi:10.1080/01621459.2014.998760>), 
  CAS (Xie et al. (2020) <doi:10.1080/01621459.2019.1573734>), 
  CI-SIS (Cheng and Wang. (2023) <doi:10.1016/j.cmpb.2022.107269>), 
  CSIS (Cheng et al. (2024) <doi:10.1007/s00180-023-01399-5>) and Log-rank SIS.
License: GPL (>= 2)
SystemRequirements: Python (>= 3.8.0)
Encoding: UTF-8
Imports: survival, MASS, Ball, reticulate, stats, crayon, cli, dr,
        foreach, parallel, doParallel, fs
Suggests: knitr, utils, pkgdown
Language: en-US
NeedsCompilation: no
Repository: CRAN
RoxygenNote: 7.2.3
Packaged: 2026-05-13 17:54:22 UTC; cxw
Date/Publication: 2026-05-13 18:50:02 UTC
Built: R 4.5.2; ; 2026-05-13 19:41:45 UTC; unix

# Generated by roxygen2: do not edit by hand

export(BcorSIS)
export(CAS)
export(CISIS)
export(CSIS)
export(Cor)
export(DCSIS)
export(GendataAFT)
export(GendataCox)
export(GendataGP)
export(GendataIC)
export(GendataIM)
export(GendataLDA)
export(GendataLGM)
export(GendataLM)
export(GendataMRM)
export(GendataPM)
export(GendataTM)
export(Kfilter)
export(Kfilter_fused)
export(Kfilter_single)
export(LogrankSIS)
export(MDCSIS)
export(MFSIS)
export(MVSIS)
export(PCSIS)
export(PSIS)
export(SIRS)
export(SIS)
export(Simdata)
export(WLS)
export(get_arccos)
export(projection_corr)
export(req_py)
import(MASS)
import(cli)
import(doParallel)
import(dr)
import(foreach)
import(parallel)
import(reticulate)
importFrom(Ball,bcorsis)
importFrom(MASS,mvrnorm)
importFrom(crayon,blue)
importFrom(crayon,cyan)
importFrom(reticulate,py_config)
importFrom(reticulate,source_python)
importFrom(stats,ks.test)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,rcauchy)
importFrom(stats,rchisq)
importFrom(stats,rexp)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,rt)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(survival,Surv)
importFrom(survival,concordancefit)
importFrom(utils,combn)

---
title: "MFSIS"
---


## Description

An implementation of popular screening methods that are commonly employed in ultra-high and high dimensional data. Through this publicly available package, we provide a unified framework to carry out model-free screening procedures including 
SIS (Fan and Lv (2008) [doi:10.1111/j.1467-9868.2008.00674.x]), 
SIRS(Zhu et al. (2011)[doi:10.1198/jasa.2011.tm10563]), 
DC-SIS (Li et al. (2012) [doi:10.1080/01621459.2012.695654]), 
MDC-SIS(Shao and Zhang (2014) [doi:10.1080/01621459.2014.887012]), 
Bcor-SIS (Pan et al. (2019) [doi:10.1080/01621459.2018.1462709]), 
PC-Screen (Liu et al. (2020) [doi:10.1080/01621459.2020.1783274]), 
WLS (Zhong et al.(2021) [doi:10.1080/01621459.2021.1918554]), 
Kfilter (Mai and Zou (2015) [doi:10.1214/14-AOS1303]),
MVSIS (Cui et al. (2015) [doi:10.1080/01621459.2014.920256]),
PSIS (Pan et al. (2016) [doi:10.1080/01621459.2014.998760]),
CAS (Xie et al. (2020) [doi:101080/01621459.2019.1573734]),
CI-SIS (Cheng and Wang (2022) [doi:10.1016/j.cmpb.2022.107269]),
CSIS (Cheng et al. (2024) [doi:10.1007/s00180-023-01399-5]) and Logrank-SIS.

## Installation
```{r install}
install.packages("MFSIS")
```

## Example
Here are many extensive examples that can let you quickly learn how to use this package. The author's original intention in writing this package is ease of use. Here is a simple example to illustrate its use. For example, I want to use SIRS method to screen high-dimensional data. I want to get the most active 30 features.
```{r SIRS}
library(MFSIS)
n=100;
p=200;
pho=0.5;
data=gendata1(n,p,pho)
data=cbind(data[[1]],data[[2]])
colnames(data)[1:ncol(data)]=c(paste0("X",1:(ncol(data)-1)),"Y")
data=as.matrix(data)
X=data[,1:(ncol(data)-1)];
Y=data[,ncol(data)];
A=SIRS(X,Y,30);A
```

#' Feature Screening via Distance Correlation Learning
#'
#' A sure independence screening procedure based on the distance correlation (DC-SIS).
#' The DC-SIS can be implemented as easily as the sure independence screening (SIS) procedure based on the Pearson correlation proposed by Fan and Lv(2008).
#' DC-SIS can be used directly to screen grouped predictor variables and multivariate response variables.
#'
#' @param X The design matrix of dimensions n * p. Each row is an observation vector.
#' @param Y The response vector of dimension n * 1.
#' @param nsis Number of predictors recruited by DCSIS. The default is n/log(n).
#'
#' @return the labels of first nsis largest active set of all predictors
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#'
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataLM(n, p, rho, error = "gaussian")
#' data <- cbind(data[[1]], data[[2]])
#' colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
#' data <- as.matrix(data)
#' X <- data[, 1:(ncol(data) - 1)]
#' Y <- data[, ncol(data)]
#' A <- DCSIS(X, Y, n / log(n))
#' A
#'
#' @references
#'
#' Fan, J. and J. Lv (2008). Sure independence screening for ultrahigh dimensional feature space. Journal of the Royal Statistical Society: Series B (Statistical Methodology) 70(5),849–911.
#'
#' Li, R., W. Zhong, and L. Zhu (2012). Feature screening via distance correlation learning. Journal of the American Statistical Association 107(499), 1129–1139.
DCSIS <- function(X, Y, nsis = (dim(X)[1]) / log(dim(X)[1])) {
  if (dim(X)[1] != length(Y)) {
    stop("X and Y should have same number of rows!")
  }
  if (missing(X) | missing(Y)) {
    stop("The data is missing!")
  }
  if (TRUE %in% (is.na(X) | is.na(Y) | is.na(nsis))) {
    stop("The input vector or matrix cannot have NA!")
  }
  if (inherits(Y, "Surv")) {
    stop("DCSIS can not implemented with object  of Surv")
  }
  n <- dim(X)[1] ## sample size
  p <- dim(X)[2] ## dimension
  B <- matrix(1, n, 1)
  C <- matrix(1, 1, p)
  sxy1 <- matrix(0, n, p)
  sxy2 <- matrix(0, n, p)
  sxy3 <- matrix(0, n, 1)
  sxx1 <- matrix(0, n, p)
  syy1 <- matrix(0, n, 1)
  for (i in 1:n) {
    XX1 <- abs(X - B %*% X[i, ])
    YY1 <- sqrt(apply((Y - B %*% Y[i])^2, 1, sum))
    sxy1[i, ] <- apply(XX1 * (YY1 %*% C), 2, mean)
    sxy2[i, ] <- apply(XX1, 2, mean)
    sxy3[i, ] <- mean(YY1)
    XX2 <- XX1^2
    sxx1[i, ] <- apply(XX2, 2, mean)
    YY2 <- YY1^2
    syy1[i, ] <- mean(YY2)
  }
  SXY1 <- apply(sxy1, 2, mean)
  SXY2 <- apply(sxy2, 2, mean) * apply(sxy3, 2, mean)
  SXY3 <- apply(sxy2 * (sxy3 %*% C), 2, mean)
  SXX1 <- apply(sxx1, 2, mean)
  SXX2 <- apply(sxy2, 2, mean)^2
  SXX3 <- apply(sxy2^2, 2, mean)
  SYY1 <- apply(syy1, 2, mean)
  SYY2 <- apply(sxy3, 2, mean)^2
  SYY3 <- apply(sxy3^2, 2, mean)
  dcovXY <- sqrt(SXY1 + SXY2 - 2 * SXY3)
  dvarXX <- sqrt(SXX1 + SXX2 - 2 * SXX3)
  dvarYY <- sqrt(SYY1 + SYY2 - 2 * SYY3)
  dcorrXY <- dcovXY / sqrt(dvarXX * dvarYY)
  A <- order(dcorrXY, decreasing = TRUE)
  return(A[1:nsis])
}

#' Generate Simulation Data for Interval-Censored Screening
#'
#' @description
#' Generates ultrahigh-dimensional simulation data for evaluating feature
#' screening methods under Case II interval censoring, following the
#' data-generating framework of the Log-rank SIS paper.
#' The true active predictor set is fixed at \eqn{\mathcal{D} = \{1,2,3,19,20\}}.
#'
#' Binary predictors are obtained by dichotomizing a latent Gaussian vector
#' with Toeplitz covariance \eqn{\Sigma_{jk} = \rho^{|j-k|}}.  The latent
#' response \eqn{Y^*} is generated from one of three models (linear, Poisson,
#' or Cox), and interval-censored observations \eqn{(Y^L_i, Y^U_i]} are
#' produced by adding asymmetric random noise to \eqn{Y^*_i}.
#'
#' @param n Number of subjects (rows of the design matrix).
#' @param p Number of predictors (columns of the design matrix).
#' @param rho AR(1) correlation between adjacent latent predictors.
#'   Must satisfy \eqn{0 \le \rho < 1}.
#' @param model Character string specifying the model for the latent response.
#'   One of \code{"linear"}, \code{"poisson"}, or \code{"cox"}.
#'   \describe{
#'     \item{\code{"linear"}}{Linear model \eqn{Y^* = \mathbf{x}^\top\boldsymbol\beta + \epsilon},
#'       \eqn{\epsilon \sim N(0,1)}.}
#'     \item{\code{"poisson"}}{Poisson regression with
#'       \eqn{\log\lambda = \mathbf{x}^\top\boldsymbol\beta},
#'       \eqn{\boldsymbol\beta = (1, 0.8, 0.6, \mathbf{0}_{15}^\top, -0.6, -0.8,
#'       \mathbf{0}_{p-20}^\top)^\top}.}
#'     \item{\code{"cox"}}{Cox proportional hazards with Weibull baseline
#'       (shape = 2, scale = 0.15) and
#'       \eqn{\boldsymbol\beta = (1.2, 1, 0.8, \mathbf{0}_{15}^\top, -1, -1.2,
#'       \mathbf{0}_{p-20}^\top)^\top}.}
#'   }
#' @param balanced Logical.  If \code{TRUE} (default), predictors are
#'   dichotomized at the median (\eqn{X_k = I(X^*_k > 0)}), yielding
#'   approximately equal group sizes.  If \code{FALSE}, the 70th percentile
#'   of \eqn{N(0,1)} is used as the threshold, producing imbalanced
#'   (sparse) binary predictors.
#' @param censoring Integer 1 or 2 selecting the censoring scenario:
#'   \describe{
#'     \item{\code{1}}{(default) \eqn{(p_I, p_R, p_L) = (80\%, 14\%, 6\%)}.}
#'     \item{\code{2}}{\eqn{(p_I, p_R, p_L) = (80\%, 6\%, 14\%)}.}
#'   }
#' @param seed Optional integer seed passed to \code{set.seed()} for
#'   reproducibility.  \code{NULL} (default) leaves the RNG state unchanged.
#'
#' @return A list with three elements:
#'   \describe{
#'     \item{\code{X}}{An \eqn{n \times p} binary integer matrix of predictors.}
#'     \item{\code{L}}{Numeric vector of length \eqn{n}: lower interval bounds.
#'       Left-censored observations have \code{L = -Inf}.}
#'     \item{\code{R}}{Numeric vector of length \eqn{n}: upper interval bounds.
#'       Right-censored observations have \code{R = Inf}.}
#'   }
#'
#' @references
#'
#' Fan, J. and Lv, J. (2008). Sure independence screening for ultrahigh
#' dimensional feature space.
#' \emph{Journal of the Royal Statistical Society: Series B},
#' \strong{70}(5), 849--911.
#'
#' Zhong, W., Qian, C., Liu, W., Zhu, L., and Li, R. (2023). Feature
#' screening for interval-valued response with application to study
#' association between posted salary and required skills.
#' \emph{Journal of the American Statistical Association},
#' \strong{118}, 805--817.
#'
#' @examples
#' ## Linear model, balanced predictors, censoring scenario 1
#' data <- GendataIC(n = 100, p = 500, rho = 0.5,
#'                   model = "linear", balanced = TRUE,
#'                   censoring = 1, seed = 2025)
#' X <- data$X; L <- data$L; R <- data$R
#' cat("Dimensions: n =", nrow(X), "  p =", ncol(X), "\n")
#' cat("Right-censored:", sum(is.infinite(R)), "\n")
#' cat("Left-censored: ", sum(is.infinite(L)), "\n")
#'
#' ## Poisson model, imbalanced predictors, censoring scenario 2
#' data2 <- GendataIC(n = 100, p = 500, rho = 0.2,
#'                    model = "poisson", balanced = FALSE,
#'                    censoring = 2, seed = 42)
#'
#' ## Cox model
#' data3 <- GendataIC(n = 100, p = 500, rho = 0.5,
#'                    model = "cox", balanced = TRUE, seed = 1)
#'
#' @importFrom MASS mvrnorm
#' @importFrom stats rpois rchisq rnorm qnorm runif rexp
#' @export
GendataIC <- function(n, p, rho,
                      model     = c("linear", "poisson", "cox"),
                      balanced  = TRUE,
                      censoring = 1L,
                      seed      = NULL) {

  model     <- match.arg(model)
  censoring <- as.integer(censoring)
  if (!censoring %in% 1:2)
    stop("'censoring' must be 1 or 2.")
  if (!is.null(seed)) set.seed(seed)

  ## ---- Active predictor set (fixed as in the paper) ----
  active_set <- c(1L, 2L, 3L, 19L, 20L)

  ## ---- Step 1: Latent continuous predictors ~ N(0, Sigma) ----
  sigma_vec <- rho^(0:(p - 1))
  Sigma     <- stats::toeplitz(sigma_vec)
  X_star    <- MASS::mvrnorm(n, mu = rep(0.0, p), Sigma = Sigma)

  ## ---- Step 2: Dichotomize to binary predictors ----
  threshold <- if (balanced) 0.0 else qnorm(0.7)
  X         <- (X_star > threshold) * 1L

  ## ---- Step 3: Generate latent response Y* ----
  a_n  <- 2.0 * log(n) / sqrt(n)   # signal magnitude (Fan & Lv 2008)

  Y_star <- switch(model,

    "linear" = {
      beta <- numeric(p)
      set_D <- c(1L, 2L, 3L)
      set_E <- c(19L, 20L)
      for (j in active_set) {
        Uj    <- if (j %in% set_D) 0L else 1L
        Zj    <- rnorm(1)
        beta[j] <- (-1)^Uj * (a_n + abs(Zj))
      }
      as.numeric(X %*% beta) + rnorm(n)
    },

    "poisson" = {
      beta <- numeric(p)
      beta[active_set] <- c(1.0, 0.8, 0.6, -0.6, -0.8)
      lambda <- exp(as.numeric(X %*% beta))
      rpois(n, lambda)
    },

    "cox" = {
      beta <- numeric(p)
      beta[active_set] <- c(1.2, 1.0, 0.8, -1.0, -1.2)
      ## Weibull baseline: shape = 2, scale = 0.15 => H0(t) = (t/0.15)^2
      lp  <- as.numeric(X %*% beta)
      U   <- runif(n)
      ## Inverse CDF: T = 0.15 * (-log(U) * exp(-lp))^(1/2)
      0.15 * (-log(U) * exp(-lp))^0.5
    }
  )

  ## ---- Step 4: Generate interval bounds (asymmetric noise) ----
  Ui  <- rpois(n, lambda = 2)          # lower noise
  Vi  <- rchisq(n, df = 2)            # upper noise
  L_i <- floor(Y_star - Ui)
  R_i <- floor(Y_star + 2.0 * Vi)

  ## ---- Step 5: Apply Case II censoring ----
  ## Censoring proportions: (p_I, p_R, p_L)
  if (censoring == 1L) {
    p_I <- 0.80; p_R <- 0.14; p_L <- 0.06
  } else {
    p_I <- 0.80; p_R <- 0.06; p_L <- 0.14
  }

  cens_type <- sample(c("I", "R", "L"), n, replace = TRUE,
                      prob = c(p_I, p_R, p_L))

  L_obs <- L_i
  R_obs <- R_i

  ## Right-censored: only lower bound observed
  idx_R        <- cens_type == "R"
  R_obs[idx_R] <- Inf

  ## Left-censored: only upper bound observed
  idx_L        <- cens_type == "L"
  L_obs[idx_L] <- -Inf

  list(X = X, L = L_obs, R = R_obs)
}

#' A Model-free Variable Screening Method Based on Leverage Score
#'
#' An innovative and effective sampling scheme based on leverage scores via singular value decompositions
#' has been proposed to select rows of a design matrix as a surrogate of the full data in linear regression.
#' Analogously, variable screening can be viewed as selecting rows of the design matrix. However, effective
#' variable selection along this line of thinking remains elusive. This method propose a
#' weighted leverage variable screening method by using both the left and right singular vectors of the design matrix.
#'
#' @param X The design matrix of dimensions n * p. Each row is an observation vector.
#' @param Y The response vector of dimension n * 1.
#' @param nsis Number of predictors recruited by WLS. The default is n/log(n).
#'
#' @return the labels of first nsis largest active set of all predictors.
#' @import dr
#'
#'
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#'
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataLM(n, p, rho, error = "gaussian")
#' data <- cbind(data[[1]], data[[2]])
#' colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
#' data <- as.matrix(data)
#' X <- data[, 1:(ncol(data) - 1)]
#' Y <- data[, ncol(data)]
#' A <- WLS(X, Y, n / log(n))
#' A
#'
#' @references
#'
#' Zhong, W., Liu, Y., & Zeng, P. (2021). A Model-free Variable Screening Method Based on Leverage Score. Journal of the American Statistical Association, (just-accepted), 1-36.
WLS <- function(X, Y, nsis = (dim(X)[1]) / log(dim(X)[1])) {
  if (dim(X)[1] != length(Y)) {
    stop("X and Y should have same number of rows!")
  }
  if (missing(X) | missing(Y)) {
    stop("The data is missing!")
  }
  if (TRUE %in% (is.na(X) | is.na(Y) | is.na(nsis))) {
    stop("The input vector or matrix cannot have NA!")
  }
  if (inherits(Y, "Surv")) {
    stop("WLS can not implemented with object  of Surv")
  }
  nslice <- 10
  cn1 <- 0.1
  cn2 <- 1
  ## basic information about X and Y
  n <- dim(X)[1]
  p <- dim(X)[2]
  h <- nslice

  ## judge whether Y is discrete
  if (length(table(Y) <= 10)) {
    Y <- as.factor(Y)
  }

  ## slice
  if (is.factor(Y) == 1) {
    index <- as.numeric(factor(Y))
    nh <- summary(factor(Y))
    h <- length(nh)
  }
  if (is.factor(Y) != 1) {
    slice <- dr.slices.arc(Y, h)
    index <- slice$slice.indicator # Slice Index
    nh <- slice$slice.sizes # Observations per Slice
  }
  ph <- nh / n

  ####################################
  ## calculate wls
  ####################################
  wls <- c() # Weighted Leverage Score
  svdx <- svd(X)
  u <- svdx$u
  d <- svdx$d
  v <- svdx$v
  dir <- min(n, p)

  ## calculate WLS
  w <- matrix(ncol = dir, nrow = length(nh))
  for (j in 1:dir) {
    for (i in 1:length(nh)) w[i, j] <- sum(u[, j] * (index == i)) / nh[i]
  }

  uut <- array(dim = c(length(nh), dir, dir)) # UUT Array
  for (i in 1:length(nh)) uut[i, , ] <- (nh[i]) * (w[i, ] %*% t(w[i, ]))

  ## LEVERAGE SCORES
  for (j in 1:p) wls[j] <- t(v[j, 1:dir]) %*% colSums(uut) %*% v[j, 1:dir]
  A <- order(wls, decreasing = T)
  return(A[1:nsis])
}

#' Model-Free Feature screening Based on Concordance Index for Ultra-High Dimensional Categorical Data
#'
#' The proposed method is based on the concordance index which measures concordance between random vectors.
#' A model-free and robust feature screening method for ultrahigh-dimensional categorical data.
#' The performance is quite robust in the presence of heavy-tailed distributions, extremely unbalance responses, and category-adaptive data.
#'
#'
#' @param X The design matrix of dimensions n * p. Each row is an observation vector.
#' @param Y The response vector of dimension n * 1.
#' @param nsis Number of predictors recruited by CISIS. The default is n/log(n).
#'
#' @return the labels of first nsis largest active set of all predictors
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#'
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataLGM(n, p, rho)
#' data <- cbind(data[[1]], data[[2]])
#' colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
#' data <- as.matrix(data)
#' X <- data[, 1:(ncol(data) - 1)]
#' Y <- data[, ncol(data)]
#' A <- CISIS(X, Y, n / log(n))
#' A
#'
#' @references
#'
#' Cheng X, Wang H. A generic model-free feature screening procedure for ultra-high dimensional data with categorical response[J]. Computer Methods and Programs in Biomedicine, 2023, 229: 107269.
CISIS <- function(X, Y, nsis) {
  if (dim(X)[1] != length(Y)) {
    stop("X and Y should have same number of rows!")
  }
  if (missing(X) | missing(Y)) {
    stop("The data is missing!")
  }
  if (TRUE %in% (is.na(X) | is.na(Y) | is.na(nsis))) {
    stop("The input vector or matrix cannot have NA!")
  }
  if (inherits(Y, "Surv")) {
    stop("SIRS can not implemented with object of Surv")
  }
  n <- dim(X)[1] ## sample size
  p <- dim(X)[2] ## dimension
  B <- vector(mode = "numeric", length = p)
  Yc <- vector(mode = "numeric", length = n)
  Yr <- unique(Y)
  if (length(Yr) > 15) {
    stop("A supposedly categorical variable was provided with more than 15 levels!")
  }
  Cindex <- matrix(0, length(Yr), p)
  for (r in 1:length(Yr)) {
    index <- which(Y == Yr[r])
    len <- length(index) * (n - length(index))
    # P=sum(Y==Yr[r])/length(Y)
    for (j in 1:p) {
      sum <- 0
      for (i in index) {
        sum <- sum + length(which(X[-index, j] < X[i, j]))
      }
      Cindex[r, j] <- abs(sum / len - 0.5)
    }
  }
  B <- apply(Cindex, 2, max)
  A <- order(B, decreasing = TRUE)
  return(A[1:nsis])
}

#' Model-free feature screening procedures
#'
#' Through this function, we provide a unified framework
#' to carry out model-free screening procedures including
#' SIS (Fan and Lv (2008) <doi:10.1111/j.1467-9868.2008.00674.x>),
#' SIRS(Zhu et al. (2011)<doi:10.1198/jasa.2011.tm10563>),
#'  DC-SIS (Li et al. (2012) <doi:10.1080/01621459.2012.695654>),
#' MDC-SIS(Shao and Zhang (2014) <doi:10.1080/01621459.2014.887012>),
#' Bcor-SIS (Pan et al. (2019) <doi:10.1080/01621459.2018.1462709>),
#'  PC-Screen (Liu et al. (2020) <doi:10.1080/01621459.2020.1783274>),
#' WLS (Zhong et al.(2021) <doi:10.1080/01621459.2021.1918554>),
#' Kfilter (Mai and Zou (2015) <doi:10.1214/14-AOS1303>),
#' MVSIS (Cui et al. (2015) <doi:10.1080/01621459.2014.920256>),
#' PSIS (Pan et al. (2016) <doi:10.1080/01621459.2014.998760>),
#'  CAS (Xie et al. (2020) <doi:101080/0162145920191573734>),
#' CI-SIS (Cheng and Wang. (2022) <doi:10.1016/j.cmpb.2022.107269>)
#' and CSIS.
#'
#' @param X The design matrix of dimensions n * p. Each row is an observation vector.
#' @param Y The response vector of dimension n * 1.
#' @param nsis Number of predictors recruited by the screening method. The default is n/log(n).
#' @param method The method that you choose to perform screening procedure.
#' method=c("SIS", "SIRS", "DCSIS", "MDCSIS", "CSIS", "PCSIS", "BcorSIS", "WLS", "MVSIS", "Kfilter","PSIS","CAS","CISIS").
#' If you want to know more information about this method, please use command "help(method)" for detail information.
#'
#' @return the labels of first nsis largest active set of all predictors
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#'
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataLM(n, p, rho, error = "gaussian")
#' data <- cbind(data[[1]], data[[2]])
#' colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
#' data <- as.matrix(data)
#' X <- data[, 1:(ncol(data) - 1)]
#' Y <- data[, ncol(data)]
#' A <- MFSIS(X, Y, n / log(n), method = "CSIS")
#' A
#'
MFSIS <- function(X, Y, nsis = (dim(X)[1]) / log(dim(X)[1]), method = c("SIS", "SIRS", "DCSIS", "MDCSIS", "CSIS", "PCSIS", "BcorSIS", "WLS", "MVSIS", "Kfilter")) {
  if (method == "SIS") { ## NO.1
    A <- SIS(X, Y, nsis)
  } else if (method == "SIRS") { ## NO.2
    A <- SIRS(X, Y, nsis)
  } else if (method == "DCSIS") { ## NO.3
    A <- DCSIS(X, Y, nsis)
  } else if (method == "MDCSIS") { ## NO.4
    A <- MDCSIS(X, Y, nsis)
  } else if (method == "CSIS") { ## NO.5
    A <- CSIS(X, Y, nsis)
  } else if (method == "BcorSIS") { ## NO.6
    A <- BcorSIS(X, Y, nsis)
  } else if (method == "WLS") { ## NO.7
    A <- WLS(X, Y, nsis)
  } else if (method == "MVSIS") { ## NO.8
    A <- MVSIS(X, Y, nsis)
  } else if (method == "PCSIS") { ## NO.9
    A <- PCSIS(X, Y, nsis)
  } else if (method == "Kfilter") { ## NO.10
    A <- Kfilter(X, Y, nsis)
  } else if (method == "PSIS") { ## NO.11
    A <- PSIS(X, Y, nsis)
  } else if (method == "CAS") { ## NO.12
    A <- CAS(X, Y, nsis)
  } else if (method == "CISIS") { ## NO.13
    A <- CISIS(X, Y, nsis)
  } else {
    stop("The author has not implemented this method yet. Please email the author!")
  }
}

#' Category-Adaptive Variable Screening for Ultra-High Dimensional Heterogeneous Categorical Data
#'
#' A category-adaptive screening procedure with high-dimensional heterogeneous data, which is to detect category-specific important covariates.
#' This proposal is a model-free approach without any specification of a regression model and an adaptive procedure
#' in the sense that the set of active variables is allowed to vary across different categories, thus making it more
#' flexible to accommodate heterogeneity.
#'
#'
#' @param X The design matrix of dimensions n * p. Each row is an observation vector.
#' @param Y The response vector of dimension n * 1.
#' @param nsis Number of predictors recruited by CAS. The default is n/log(n).
#'
#' @return the labels of first nsis largest active set of all predictors
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#'
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataLGM(n, p, rho)
#' data <- cbind(data[[1]], data[[2]])
#' colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
#' data <- as.matrix(data)
#' X <- data[, 1:(ncol(data) - 1)]
#' Y <- data[, ncol(data)]
#' A <- CAS(X, Y, n / log(n))
#' A
#'
#' @references
#'
#' Pan, R., Wang, H., and Li, R. (2016). Ultrahigh-dimensional multiclass linear discriminant analysis by pairwise sure independence screening. Journal of the American Statistical Association, 111(513):169–179.
CAS <- function(X, Y, nsis) {
  if (dim(X)[1] != length(Y)) {
    stop("X and Y should have same number of rows!")
  }
  if (missing(X) | missing(Y)) {
    stop("The data is missing!")
  }
  if (TRUE %in% (is.na(X) | is.na(Y) | is.na(nsis))) {
    stop("The input vector or matrix cannot have NA!")
  }
  if (inherits(Y, "Surv")) {
    stop("SIRS can not implemented with object of Surv")
  }
  Yr <- unique(Y)
  if (length(Yr) > 15) {
    stop("A supposedly categorical variable was provided with more than 15 levels!")
  }
  n <- dim(X)[1]
  p <- dim(X)[2]
  cas <- matrix(0, p, length(Yr))
  for (r in 1:length(Yr)) {
    index <- which(Y == Yr[r])
    tau <- 0
    for (i in index) {
      for (k in 1:n) {
        tau <- tau + (X[i, ] <= X[k, ])
      }
    }
    cas[, r] <- abs(tau / (n + 1) / sum(Y == Yr[r]) - 1 / 2)
  }
  B <- apply(cas, 1, max)
  A <- order(B, decreasing = TRUE)
  return(A[1:nsis])
}

#' Generate simulation data (Multivariate response models)
#'
#' This function helps you quickly generate simulation data based on transformation model.
#' You just need to input the sample and dimension of the data
#' you want to generate and the covariance parameter pho.
#' This simulated example comes from Example 3 introduced by Li et al.(2020)
#'
#' @param n Number of subjects in the dataset to be simulated. It will also equal to the
#' number of rows in the dataset to be simulated, because it is assumed that each
#' row represents a different independent and identically distributed subject.
#' @param p Number of predictor variables (covariates) in the simulated dataset.
#' These covariates will be the features screened by model-free procedures.
#' @param rho The correlation between adjacent covariates in the simulated matrix X.
#' The within-subject covariance matrix of X is assumed to has the same form as an
#' AR(1) auto-regressive covariance matrix, although this is not meant to imply
#' that the X covariates for each subject are in fact a time series. Instead, it is just
#' used as an example of a parsimonious but nontrivial covariance structure. If
#' rho is left at the default of zero, the X covariates will be independent and the
#' simulation will run faster.
#' @param type The type of multivariate response models, which use different mean and covariance
#' structure to generate data. Specially, type="a" is following the Model 3.a and
#' type="b" is following the Model 3.b by Li et al.(2020).
#' @return the list of your simulation data
#' @import MASS
#' @importFrom MASS mvrnorm
#' @importFrom stats rnorm
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataMRM(n, p, rho, type = "a")
#'
#' @references
#'
#' Liu, W., Y. Ke, J. Liu, and R. Li (2020). Model-free feature screening and FDR control with knockoff features. Journal of the American Statistical Association, 1–16.
GendataMRM <- function(n, p, rho,
                       type = c("a", "b")) # n sample size; p dimension size.
{
  sig1 <- matrix(0, p, p)
  sig1 <- rho^abs(row(sig1) - col(sig1))
  diag(sig1) <- rep(1, p)
  X <- mvrnorm(n, rep(0, p), sig1)
  if (type != "a" & type != "b") {
    stop("The type can be implemented both a and b ")
  }
  if (type == "a") {
    Y <- matrix(0, n, 2)
    for (i in 1:n) {
      mu1 <- exp(2 * (X[i, 1] + X[i, 2]))
      mu2 <- X[i, 3] + X[i, 4] + X[i, 5]
      sig2 <- matrix(0, 2, 2)
      cita <- 2 * X[i, 1] + 2 * X[i, 2] + 2 * X[i, 3] + 2 * X[i, 4] + 2 * X[i, 5]
      sig2[1, 2] <- sin(cita)
      sig2[2, 1] <- sig2[1, 2]
      diag(sig2) <- rep(1, 2)
      Y[i, ] <- mvrnorm(1, c(mu1, mu2), sig2)
    }
  } else {
    Y <- matrix(0, n, 2)
    for (i in 1:n) {
      mu1 <- 2 * sin(pi * X[i, 1] / 2) + X[i, 3] + exp(1 + X[i, 5])
      mu2 <- 1 / X[i, 2] + X[i, 4]
      sig2 <- matrix(0, 2, 2)
      cita <- 2 * X[i, 1] + 2 * X[i, 2] + 2 * X[i, 3] + 2 * X[i, 4] + 2 * X[i, 5]
      sig2[1, 2] <- (exp(cita) - 1) / (exp(cita) + 1)
      sig2[2, 1] <- sig2[1, 2]
      diag(sig2) <- rep(1, 2)
      Y[i, ] <- mvrnorm(1, c(mu1, mu2), sig2)
    }
  }
  return(list(X = X, Y = Y))
}

#' The Kolmogorov filter for variable screening
#'
#' A new model-free screening method called the fused Kolmogorov filter is proposed for high-dimensional data analysis.
#' This new method is fully nonparametric and can work with many types of covariates and response variables, including continuous,
#' discrete and categorical variables.
#'
#' @param X The design matrix of dimensions n * p. Each row is an observation vector.
#' @param Y The response vector of dimension n * 1.
#' @param nsis Number of predictors recruited by SIS. The default is n/log(n).
#'
#' @return the labels of first nsis largest active set of all predictors
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#'
#'n=100;
#'p=200;
#'rho=0.5;
#'data=GendataLM(n,p,rho,error="gaussian")
#'data=cbind(data[[1]],data[[2]])
#'colnames(data)[1:ncol(data)]=c(paste0("X",1:(ncol(data)-1)),"Y")
#'data=as.matrix(data)
#'X=data[,1:(ncol(data)-1)];
#'Y=data[,ncol(data)];
#'A=Kfilter(X,Y,n/log(n));A
#'
#' @references
#'
#' Mai, Q., & Zou, H. (2013). The Kolmogorov filter for variable screening in high-dimensional binary classification. Biometrika, 100(1), 229-234.
#'
#' Mai, Q., & Zou, H. (2015). The fused Kolmogorov filter: A nonparametric model-free screening method. The Annals of Statistics, 43(4), 1471-1497.
Kfilter<-function(X,Y,nsis=(dim(X)[1])/log(dim(X)[1])){
  if (dim(X)[1]!=length(Y)) {
    stop("X and Y should have same number of rows!")
  }
  if (missing(X)|missing(Y)) {
    stop("The data is missing!")
  }
  if (TRUE%in%(is.na(X)|is.na(Y)|is.na(nsis))) {
    stop("The input vector or matrix cannot have NA!")
  }
  if (inherits(Y,"Surv")) {
    stop("Kfilter can not implemented with object  of Surv")
  }
  N=NULL;
  nslices=NULL;
  slicing.scheme=NULL;

  if(length(table(Y))<=10){
    Y<-factor(Y)
    obj<-Kfilter_single(X=X,Y=Y,nsis)
    }else{
      n<-nrow(X)
      if(is.null(N)){N<-ceiling(log(n))-2}
      if(is.null(nslices)){nslices<-3:(N+2)}
      obj<-Kfilter_fused(X=X,Y=Y,nsis)
    }
  return (obj)
}


#' The Kolmogorov filter for variable screening in high-dimensional binary classification
#'
#' @param X The design matrix of dimensions n * p. Each row is an observation vector.
#' @param Y The response vector of dimension n * 1.
#' @param nsis Number of predictors recruited by Kfilter_single. The default is n/log(n).
#'
#' @return the labels of first nsis largest active set of all predictors
#' @importFrom stats quantile
#' @importFrom stats ks.test
#' @export
#'
#' @examples
#' n=100;
#' p=200;
#' rho=0.5;
#' data=GendataLGM(n,p,rho)
#' data=cbind(data[[1]],data[[2]])
#' colnames(data)[1:ncol(data)]=c(paste0("X",1:(ncol(data)-1)),"Y")
#' data=as.matrix(data)
#' X=data[,1:(ncol(data)-1)];
#' Y=data[,ncol(data)];
#' A=Kfilter_single(X,Y,n/log(n));A
#'
#' @references
#' #' Mai, Q., & Zou, H. (2013). The Kolmogorov filter for variable screening in high-dimensional binary classification. Biometrika, 100(1), 229-234.
Kfilter_single<-function(X,Y,nsis=(dim(X)[1])/log(dim(X)[1])){
  Y=as.factor(Y)
  Y.dm<-Y
  Y.dm<-as.numeric(Y.dm)
  K<-length(unique(Y.dm))
  p<-ncol(X)
  ks.stat<-matrix(0,p,K*(K-1)/2)

  nclass<-0
  for(j in 1:(K-1)){
    for(l in (j+1):K){
      nclass<-nclass+1
      for(i in 1:p){
        ks.stat[i,nclass]<-ks.test(X[Y.dm==j,i],X[Y.dm==l,i])$statistic}
    }
  }
  ks.stat.max0<-apply(ks.stat,1,max)
  k.rank<-rank(-ks.stat.max0,ties.method="first")
  return (k.rank[1:nsis])
}


#' The fused kolmogorov filter: a nonparametric model-free screening method
#'
#' @param X The design matrix of dimensions n * p. Each row is an observation vector.
#' @param Y The response vector of dimension n * 1.
#' @param nsis Number of predictors recruited by Kfilter_fused. The default is n/log(n).
#'
#' @return the labels of first nsis largest active set of all predictors
#' @importFrom stats quantile
#' @importFrom stats ks.test
#' @export
#'
#' @examples
#' ##Scenario 1  generate discrete response data
#' n=100;
#' p=200;
#' R=5;
#' data=GendataLDA(n,p,R,error="gaussian",style="balanced")
#' data=cbind(data[[1]],data[[2]])
#' colnames(data)[1:ncol(data)]=c(paste0("X",1:(ncol(data)-1)),"Y")
#' data=as.matrix(data)
#' X=data[,1:(ncol(data)-1)];
#' Y=data[,ncol(data)];
#' A1=Kfilter_fused(X,Y,n/log(n));A1
#'
#' ##Scenario 2  generate continuous response data
#' n=50;
#' p=200;
#' rho=0.5;
#' data=GendataLM(n,p,rho,error="gaussian")
#' data=cbind(data[[1]],data[[2]])
#' colnames(data)[1:ncol(data)]=c(paste0("X",1:(ncol(data)-1)),"Y")
#' data=as.matrix(data)
#' X=data[,1:(ncol(data)-1)];
#' Y=data[,ncol(data)];
#' A2=Kfilter_fused(X,Y,n/log(n));A2
#'
#' @references
#'  Mai, Q., & Zou, H. (2015). The fused Kolmogorov filter: A nonparametric model-free screening method. The Annals of Statistics, 43(4), 1471-1497.
Kfilter_fused<-function(X,Y,nsis=(dim(X)[1])/log(dim(X)[1])){

  n<-nrow(X)
  p<-ncol(X)

  N<-ceiling(log(n))-2
  nslices<-3:(N+2)

  ks.stat.single<-matrix(0,N,p)
  ks.stat.max<-rep(0,p)

  for(K in nslices){
    slicing.scheme<-quantile(Y,seq(0,1,1/K))
    slicing.scheme[1]<-slicing.scheme[1]-1
    slicing.scheme[K+1]<-slicing.scheme[K+1]+1
    Y.dm<-cut(Y,slicing.scheme,labels=c(1:K),right=F)
    ks.stat<-matrix(0,p,K*(K-1)/2)

    nclass<-0
    for(j in 1:(K-1)){
      for(l in (j+1):K){
        nclass<-nclass+1
        for(i in 1:p){
          ks.stat[i,nclass]<-ks.test(X[Y.dm==j,i],X[Y.dm==l,i])$statistic}
      }
    }
    ks.stat.max0<-apply(ks.stat,1,max)
    ks.stat.single[K-2,]<-ks.stat.max0
    ks.stat.max<-ks.stat.max+ks.stat.max0
  }
  k.rank=rank(-ks.stat.max,ties.method="first")
  return (k.rank[1:nsis])
}

#' Generate simulation data (Complete data based on transformation model)
#'
#' This function helps you quickly generate simulation data based on transformation model.
#' You just need to input the sample and dimension of the data
#' you want to generate and the covariance parameter pho.
#' This simulated example comes from Example 3.a introduced by Zhu et al.(2011)
#'
#' @param n Number of subjects in the dataset to be simulated. It will also equal to the
#' number of rows in the dataset to be simulated, because it is assumed that each
#' row represents a different independent and identically distributed subject.
#' @param p Number of predictor variables (covariates) in the simulated dataset.
#' These covariates will be the features screened by model-free procedures.
#' @param rho The correlation between adjacent covariates in the simulated matrix X.
#' The within-subject covariance matrix of X is assumed to has the same form as an
#' AR(1) auto-regressive covariance matrix, although this is not meant to imply
#' that the X covariates for each subject are in fact a time series. Instead, it is just
#' used as an example of a parsimonious but nontrivial covariance structure. If
#' rho is left at the default of zero, the X covariates will be independent and the
#' simulation will run faster.
#' @param beta A vector with length of n, which are the coefficients that you want to generate
#' about Linear model. The default is beta=(1,1,1,1,1,0,...,0)^T;
#' @param error The distribution of error term.
#'
#' @return the list of your simulation data
#' @import MASS
#' @importFrom MASS mvrnorm
#' @importFrom stats rnorm
#' @importFrom stats rt
#' @importFrom stats rcauchy
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataTM(n, p, rho, error = "gaussian")
#'
#' @references
#'
#' Zhu, L.-P., L. Li, R. Li, and L.-X. Zhu (2011). Model-free feature screening for ultrahigh-dimensional data. Journal of the American Statistical Association 106(496), 1464–1475.
GendataTM <- function(n, p, rho,
                      beta = c(rep(1, 5), rep(0, p - 5)),
                      error = c("gaussian", "t", "cauchy")) # n sample size; p dimension size.
{
  sig <- matrix(0, p, p)
  sig <- rho^abs(row(sig) - col(sig))
  diag(sig) <- rep(1, p)
  X <- mvrnorm(n, rep(0, p), sig)
  myrates <- exp(X %*% beta)
  if (error == "gaussian" | is.null(error)) {
    Y <- myrates * exp(rnorm(n))
  } else if (error == "t") {
    Y <- myrates * exp(rt(n, 2))
  } else if (error == "cauchy") {
    Y <- myrates * exp(rcauchy(n))
  } else {
    stop("The author has not implemented this error term yet.")
  }
  return(list(X = X, Y = Y))
}

#' A Generic Sure Independence Screening Procedure
#'
#' A generic nonparametric sure independence screening procedure,
#' called BCor-SIS, on the basis of a recently developed universal
#' dependence measure: Ball correlation.
#' We show that the proposed procedure has strong screening consistency even
#' when the dimensionality is an exponential order of the sample size
#' without imposing sub-exponential moment assumptions on the data.
#'
#' @param X The design matrix of dimensions n * p. Each row is an observation vector.
#' @param Y The response vector of dimension n * 1. For survival models, Y should
#' be an object of class Surv, as provided by the function
#' Surv() in the package survival.
#' @param nsis Number of predictors recruited by BcorSIS. The default is n/log(n).
#'
#' @return the labels of first nsis largest active set of all predictors
#'
#' @importFrom Ball bcorsis
#' @importFrom survival Surv
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#'
#' ## Scenario 1  generate complete data
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataLM(n, p, rho, error = "gaussian")
#' data <- cbind(data[[1]], data[[2]])
#' colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
#' data <- as.matrix(data)
#' X <- data[, 1:(ncol(data) - 1)]
#' Y <- data[, ncol(data)]
#' A1 <- BcorSIS(X, Y, n / log(n))
#' A1
#'
#' ## Scenario 2  generate survival data
#' library(survival)
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataCox(n, p, rho)
#' data <- cbind(data[[1]], data[[2]], data[[3]])
#' colnames(data)[ncol(data)] <- c("status")
#' colnames(data)[(ncol(data) - 1)] <- c("time")
#' colnames(data)[(1:(ncol(data) - 2))] <- c(paste0("X", 1:(ncol(data) - 2)))
#' data <- as.matrix(data)
#' X <- data[, 1:(ncol(data) - 2)]
#' Y <- Surv(data[, (ncol(data) - 1)], data[, ncol(data)])
#' A2 <- BcorSIS(X, Y, n / log(n))
#' A2
#' @references
#'
#' Pan, W., X. Wang, H. Zhang, H. Zhu, and J. Zhu (2020). Ball covariance: A generic measure of dependence in banach space. Journal of the American Statistical Association 115(529),307–317.
#'
#' Pan, W., X. Wang, W. Xiao, and H. Zhu (2019). A generic sure independence screening procedure. Journal of the American Statistical Association 114(526), 928–937.
BcorSIS <- function(X, Y, nsis = (dim(X)[1]) / log(dim(X)[1])) {
  if (dim(X)[1] != length(Y)) {
    stop("X and Y should have same number of rows!")
  }
  if (missing(X) | missing(Y)) {
    stop("The data is missing!")
  }
  if (TRUE %in% (is.na(X) | is.na(Y) | is.na(nsis))) {
    stop("The input vector or matrix cannot have NA!")
  }
  p <- dim(X)[2] ## dimension
  if (class(Y) == c("Surv")) {
    model <- bcorsis(x = X, y = Y, method = "survival", d = p)
    A <- model$ix
  } else {
    model <- bcorsis(x = X, y = Y, method = "standard", d = p)
    A <- model$ix
  }
  return(A[1:nsis])
}

#' Model-Free Feature screening Based on Concordance Index Statistic
#'
#' A model-free and data-adaptive feature screening method for
#' ultrahigh-dimensional data and even survival data. The proposed method is based
#' on the concordance index which measures concordance between random vectors even
#' if one of the vectors is a survival object Surv. This rank correlation based
#' method does not require specifying a regression model, and applies robustly to data
#' in the presence of censoring and heavy tails. It enjoys both sure screening and rank
#' consistency properties under weak assumptions.
#'
#' @param X The design matrix of dimensions n * p. Each row is an observation vector.
#' @param Y The response vector of dimension n * 1. For survival models,
#' Y should be an object of class Surv, as provided by the function
#' Surv() in the package survival.
#' @param nsis Number of predictors recruited by CSIS. The default is n/log(n).
#'
#' @return the labels of first nsis largest active set of all predictors
#'
#' @importFrom survival concordancefit
#' @importFrom survival Surv
#' @import foreach
#' @import parallel
#' @import doParallel
#'
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#'
#' ## Scenario 1  generate complete data
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataLM(n, p, rho, error = "gaussian")
#' data <- cbind(data[[1]], data[[2]])
#' colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
#' data <- as.matrix(data)
#' X <- data[, 1:(ncol(data) - 1)]
#' Y <- data[, ncol(data)]
#' A1 <- CSIS(X, Y, n / log(n))
#' A1
#'
#' ## Scenario 2  generate survival data
#' library(survival)
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataCox(n, p, rho)
#' data <- cbind(data[[1]], data[[2]], data[[3]])
#' colnames(data)[ncol(data)] <- c("status")
#' colnames(data)[(ncol(data) - 1)] <- c("time")
#' colnames(data)[(1:(ncol(data) - 2))] <- c(paste0("X", 1:(ncol(data) - 2)))
#' data <- as.matrix(data)
#' X <- data[, 1:(ncol(data) - 2)]
#' Y <- Surv(data[, (ncol(data) - 1)], data[, ncol(data)])
#' A2 <- CSIS(X, Y, n / log(n))
#' A2
#'
#' @references
#'
#' Cheng X, Li G, Wang H. The concordance filter: an adaptive model-free feature screening procedure[J]. Computational Statistics, 2024, 39:2413–2436.
CSIS <- function(X, Y, nsis = (dim(X)[1]) / log(dim(X)[1])) {
  if (dim(X)[1] != length(Y)) {
    stop("X and Y should have same number of rows!")
  }
  if (missing(X) | missing(Y)) {
    stop("The data is missing!")
  }
  if (TRUE %in% (is.na(X) | is.na(Y) | is.na(nsis))) {
    stop("The input vector or matrix cannot have NA!")
  }
  n <- dim(X)[1] ## sample size
  p <- dim(X)[2] ## dimension
  B <- vector(mode = "numeric", length = p)
  Cindex <- vector(mode = "numeric", length = p)
  if (n * p <= 2000000) {
    for (i in 1:p) {
      Cindex[i] <- concordancefit(Y, X[, i])$concordance
    }
  } else {
    # Real physical cores in the computer
    cores <- detectCores(logical = FALSE)
    cl <- makeCluster(cores)
    registerDoParallel(cl, cores = cores)
    j <- NULL
    Cindex <- foreach::foreach(
      j = 1:p, .combine = "c",
      .packages = c("survival")
    ) %dopar%
      concordancefit(Y, X[, j])$concordance
    stopImplicitCluster()
    stopCluster(cl)
  }
  num <- which(Cindex < 0.5)
  B[num] <- 1 - Cindex[num]
  B[-num] <- Cindex[-num]
  A <- order(B, decreasing = TRUE)
  return(A[1:nsis])
}

#' Model-Free Feature Screening for Ultrahigh Dimensional Data
#'
#' A novel feature screening procedure under a unified model framework,
#' which covers a wide variety of commonly used parametric and semi-parametric models.
#' This method does not require imposing a specific model structure on regression functions,
#' and thus is particularly appealing to ultrahigh-dimensional regressions, where there are a
#' huge number of candidate predictors but little information about the actual model forms.
#'
#'
#' @param X The design matrix of dimensions n * p. Each row is an observation vector.
#' @param Y The response vector of dimension n * 1.
#' @param nsis Number of predictors recruited by SIRS. The default is n/log(n).
#'
#' @return the labels of first nsis largest active set of all predictors
#' @importFrom stats sd
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#'
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataLM(n, p, rho, error = "gaussian")
#' data <- cbind(data[[1]], data[[2]])
#' colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
#' data <- as.matrix(data)
#' X <- data[, 1:(ncol(data) - 1)]
#' Y <- data[, ncol(data)]
#' A <- SIRS(X, Y, n / log(n))
#' A
#'
#' @references
#'
#' Zhu, L.-P., L. Li, R. Li, and L.-X. Zhu (2011). Model-free feature screening for ultrahigh-dimensional data. Journal of the American Statistical Association 106(496), 1464–1475.
#'
SIRS <- function(X, Y, nsis = (dim(X)[1]) / log(dim(X)[1])) {
  if (dim(X)[1] != length(Y)) {
    stop("X and Y should have same number of rows!")
  }
  if (missing(X) | missing(Y)) {
    stop("The data is missing!")
  }
  if (TRUE %in% (is.na(X) | is.na(Y) | is.na(nsis))) {
    stop("The input vector or matrix cannot have NA!")
  }
  if (inherits(Y, "Surv")) {
    stop("SIRS can not implemented with object of Surv")
  }
  n <- dim(X)[1] ## sample size
  posit <- order(Y, decreasing = FALSE)
  Y <- Y[posit]
  X <- X[posit, ]
  xx <- (X - as.matrix(rep(1, n)) %*% apply(X, 2, mean)) / (as.matrix(rep(1, n)) %*% apply(X, 2, sd))
  B <- matrix(1, n, n)
  B[!upper.tri(B, diag = TRUE)] <- 0
  USIRS <- apply((t(xx) %*% B / n)^2, 1, mean)
  A <- order(USIRS, decreasing = TRUE)
  B <- A[1:nsis]
  return(B)
}

# =============================================================================
# Internal helpers (not exported)
# =============================================================================

#' Turnbull EM algorithm for NPMLE of interval-censored data
#'
#' @param l Numeric vector of lower bounds.
#' @param r Numeric vector of upper bounds (Inf for right-censored).
#' @param supp Numeric vector of support points.
#' @param max_iter Maximum number of EM iterations.
#' @param tol Convergence tolerance.
#' @return Numeric vector of probability masses on \code{supp}.
#' @keywords internal
turnbull_em <- function(l, r, supp, max_iter = 1000, tol = 1e-10) {
  n <- length(l)
  J <- length(supp)

  ## Build the indicator matrix alpha[i,j] = 1 iff supp[j] in observed interval of i
  alpha <- matrix(0.0, n, J)
  for (i in seq_len(n)) {
    li <- l[i]; ri <- r[i]
    if      ( is.finite(li) &&  is.finite(ri)) alpha[i, supp >  li & supp <= ri] <- 1.0
    else if ( is.finite(li) && !is.finite(ri)) alpha[i, supp >  li              ] <- 1.0
    else if (!is.finite(li) &&  is.finite(ri)) alpha[i,               supp <= ri] <- 1.0
  }

  covered <- colSums(alpha) > 0
  if (!any(covered)) return(rep(0.0, J))

  alpha_c <- alpha[, covered, drop = FALSE]
  Jc      <- sum(covered)
  p       <- rep(1.0 / Jc, Jc)

  for (iter in seq_len(max_iter)) {
    p_old <- p
    denom <- as.vector(alpha_c %*% p)
    denom[denom < 1e-14] <- 1e-14
    w     <- sweep(alpha_c * matrix(p, n, Jc, byrow = TRUE), 1, denom, "/")
    p     <- colMeans(w)
    if (max(abs(p - p_old)) < tol) break
  }

  p_full          <- numeric(J)
  p_full[covered] <- p
  p_full
}


#' Precompute global log-rank quantities shared across all p features
#'
#' Fits a single pooled NPMLE under the null (no covariate effect) and
#' computes posterior weights \eqn{w_{ij}}, pooled pseudo-failure counts
#' \eqn{\hat{d}_j}, and pooled pseudo-risk-set sizes \eqn{\hat{n}_j}.
#' These are reused for every feature at negligible additional cost.
#'
#' @param l Numeric vector of lower bounds.
#' @param r Numeric vector of upper bounds.
#' @return A list with components \code{w}, \code{d_j}, \code{n_j}, \code{n}.
#' @keywords internal
logrank_globals <- function(l, r) {
  n    <- length(l)
  supp <- sort(unique(c(l[is.finite(l)], r[is.finite(r)])))

  p_hat   <- turnbull_em(l, r, supp)
  active  <- p_hat > 1e-14
  supp_a  <- supp[active]
  p_hat_a <- p_hat[active] / sum(p_hat[active])
  Ja      <- length(supp_a)

  ## Re-build alpha on the active support only
  alpha <- matrix(0L, n, Ja)
  for (i in seq_len(n)) {
    li <- l[i]; ri <- r[i]
    if      ( is.finite(li) &&  is.finite(ri)) alpha[i, supp_a >  li & supp_a <= ri] <- 1L
    else if ( is.finite(li) && !is.finite(ri)) alpha[i, supp_a >  li               ] <- 1L
    else if (!is.finite(li) &&  is.finite(ri)) alpha[i,                supp_a <= ri] <- 1L
  }

  denom <- as.vector(alpha %*% p_hat_a)
  denom[denom < 1e-14] <- 1e-14
  w   <- sweep(alpha * matrix(p_hat_a, n, Ja, byrow = TRUE), 1, denom, "/")
  d_j <- colSums(w)
  n_j <- rev(cumsum(rev(d_j)))

  list(w = w, d_j = d_j, n_j = n_j, n = n)
}


#' Variance-adjusted generalized log-rank utility for one binary feature
#'
#' Computes \eqn{\hat\omega_k = |\hat U_k| / [n \hat\pi_k (1 - \hat\pi_k)]}
#' using precomputed global quantities from \code{logrank_globals()}.
#'
#' @param x_k Integer vector of length n with values in \{0, 1\}.
#' @param g List returned by \code{logrank_globals()}.
#' @return A single non-negative numeric value.
#' @keywords internal
logrank_score <- function(x_k, g) {
  ## Skip features with fewer than 2 observations in either group
  if (sum(x_k) < 2L || sum(1L - x_k) < 2L) return(0.0)

  d_jk <- colSums(x_k * g$w)
  n_jk <- rev(cumsum(rev(d_jk)))
  ok   <- g$n_j > 1e-9
  if (!any(ok)) return(0.0)

  ## Generalized log-rank score U_k / n  (Eq. 1 / n in the paper)
  U_n  <- sum(d_jk[ok] - g$d_j[ok] * (n_jk[ok] / g$n_j[ok])) / g$n

  ## Variance adjustment by marginal prevalence (Eq. 2 in the paper)
  pi_k <- mean(x_k)
  abs(U_n) / (pi_k * (1.0 - pi_k) + 1e-8)
}


# =============================================================================
# Exported function
# =============================================================================

#' Generalized Log-rank Sure Independence Screening for Ultrahigh-Dimensional
#' Interval-Censored Response
#'
#' @description
#' A computationally efficient and statistically robust feature screening
#' procedure for ultrahigh-dimensional Case II interval-censored responses
#' with binary predictors (e.g., SNP indicators or text-derived skill words).
#'
#' The marginal utility is the \emph{variance-adjusted generalized log-rank
#' statistic}
#' \deqn{\hat\omega_k = \frac{|\hat U_k|}{n\,\hat\pi_k(1-\hat\pi_k)},}
#' where \eqn{\hat U_k} is the generalized log-rank score of Sun (1996)
#' evaluated at the pooled NPMLE \eqn{\hat{\mathbf p}} fitted once to the
#' full sample under the null hypothesis of no covariate effect.  Because
#' \eqn{\hat{\mathbf p}} and the resulting posterior weights \eqn{w_{ij}} are
#' shared across all \eqn{p} features, evaluating each feature reduces to
#' simple vector operations, achieving dramatic speed-ups over ADD-SIS
#' (Zhong et al., 2023), which requires \eqn{2p} stratified NPMLE fits.
#'
#' @param X The design matrix of dimensions \eqn{n \times p}.  Each row is an
#'   observation vector.  Predictors should be binary (0/1), as commonly
#'   arising from SNP minor-allele encoding or word-segmentation of job
#'   postings.
#' @param L A numeric vector of length \eqn{n} giving the \strong{lower}
#'   endpoints of the observed intervals \eqn{(L_i, R_i]}.  For left-censored
#'   observations (event known to have occurred no later than \eqn{R_i}), set
#'   \code{L[i] = 0} or \code{L[i] = -Inf} as appropriate for the application.
#' @param R A numeric vector of length \eqn{n} giving the \strong{upper}
#'   endpoints of the observed intervals.  Set \code{R[i] = Inf} for
#'   right-censored observations (event not yet observed by the end of
#'   follow-up).
#' @param nsis Number of predictors recruited by \code{LogrankSIS}.  The
#'   default is \code{floor(n / log(n))}.
#'
#' @return An integer vector of length \code{nsis} containing the
#'   \strong{column indices} of the top-ranked predictors in \code{X},
#'   ordered by decreasing marginal utility \eqn{\hat\omega_k}.
#'
#' @references
#'
#' Sun, J. (1996). A non-parametric test for interval-censored failure time
#' data with application to AIDS studies.
#' \emph{Statistics in Medicine}, \strong{15}, 1387--1395.
#'
#' Zhao, Q. and Sun, J. (2004). Generalized log-rank test for mixed
#' interval-censored failure time data.
#' \emph{Statistics in Medicine}, \strong{23}(10), 1621--1629.
#'
#' Zhong, W., Qian, C., Liu, W., Zhu, L., and Li, R. (2023). Feature
#' screening for interval-valued response with application to study
#' association between posted salary and required skills.
#' \emph{Journal of the American Statistical Association},
#' \strong{118}, 805--817.
#'
#' Fan, J. and Lv, J. (2008). Sure independence screening for ultrahigh
#' dimensional feature space.
#' \emph{Journal of the Royal Statistical Society: Series B},
#' \strong{70}(5), 849--911.
#'
#' @examples
#' ## ---- Example 1: Balanced binary predictors, linear model ----
#' set.seed(2025)
#' n   <- 200
#' p   <- 1000
#' rho <- 0.5
#'
#' ## Toeplitz covariance for latent continuous predictors
#' data <- GendataIC(n, p, rho, model = "linear", balanced = TRUE)
#' X <- data[[1]]   # n x p binary design matrix
#' L <- data[[2]]   # lower bounds
#' R <- data[[3]]   # upper bounds (Inf = right-censored)
#'
#' ## Run LogrankSIS (active predictors are indices 1, 2, 3, 19, 20)
#' result <- LogrankSIS(X, L, R, nsis = floor(n / log(n)))
#' cat("Active predictors recovered:", intersect(result, c(1, 2, 3, 19, 20)), "\n")
#'
#' ## ---- Example 2: Imbalanced binary predictors, Cox model ----
#' set.seed(2025)
#' data2 <- GendataIC(n, p, rho, model = "cox", balanced = FALSE)
#' X2 <- data2[[1]]; L2 <- data2[[2]]; R2 <- data2[[3]]
#'
#' result2 <- LogrankSIS(X2, L2, R2, nsis = floor(n / log(n)))
#' cat("Active predictors recovered:", intersect(result2, c(1, 2, 3, 19, 20)), "\n")
#'
#' @export
LogrankSIS <- function(X, L, R,
                       nsis = floor(dim(X)[1] / log(dim(X)[1]))) {

  ## ---- Input validation ----
  X <- as.matrix(X)
  n <- nrow(X)
  p <- ncol(X)
  L <- as.numeric(L)
  R <- as.numeric(R)

  if (length(L) != n)
    stop("'L' must have length equal to nrow(X).")
  if (length(R) != n)
    stop("'R' must have length equal to nrow(X).")
  if (any(L > R, na.rm = TRUE))
    stop("All lower bounds 'L' must be <= corresponding upper bounds 'R'.")
  if (nsis > p) {
    warning("'nsis' exceeds number of predictors; setting nsis = p.")
    nsis <- p
  }

  ## ---- Step 1: Single global NPMLE (computed once for all p features) ----
  g <- logrank_globals(L, R)

  ## ---- Step 2: Variance-adjusted log-rank utility for each feature ----
  omega <- vapply(seq_len(p),
                  function(k) logrank_score(X[, k], g),
                  numeric(1L))

  ## ---- Step 3: Return indices of top nsis predictors ----
  order(omega, decreasing = TRUE)[seq_len(nsis)]
}

#' Sure Independent Screening
#'
#' To overcome challenges caused by ultra-high dimensionality,
#' Fan and Lv (2008) proposed a sure independence screening (SIS)
#' method, which aims to screen out the redundant features by
#' ranking their marginal Pearson correlations. The SIS method
#' is named after the SIS property, which states the selected subset
#' of features contains all the active ones with probability
#' approaching one.
#'
#' @param X The design matrix of dimensions n * p. Each row is an observation vector.
#' @param Y The response vector of dimension n * 1.
#' @param nsis Number of predictors recruited by SIS. The default is n/log(n).
#'
#' @return the labels of first nsis largest active set of all predictors
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#'
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataLM(n, p, rho, error = "gaussian")
#' data <- cbind(data[[1]], data[[2]])
#' colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
#' data <- as.matrix(data)
#' X <- data[, 1:(ncol(data) - 1)]
#' Y <- data[, ncol(data)]
#' A <- SIS(X, Y, n / log(n))
#' A
#'
#' @references
#'
#' Fan, J. and J. Lv (2008). Sure independence screening for ultrahigh dimensional feature space. Journal of the Royal Statistical Society: Series B (Statistical Methodology) 70(5),849–911.
SIS <- function(X, Y, nsis = (dim(X)[1]) / log(dim(X)[1])) {
  if (dim(X)[1] != length(Y)) {
    stop("X and Y should have same number of rows!")
  }
  if (missing(X) | missing(Y)) {
    stop("The data is missing!")
  }
  if (TRUE %in% (is.na(X) | is.na(Y) | is.na(nsis))) {
    stop("The input vector or matrix cannot have NA!")
  }
  if (inherits(Y, "Surv")) {
    stop("SIS can not implemented with object  of Surv")
  }
  A <- order(abs(t(X) %*% Y), decreasing = TRUE)
  return(A[1:nsis])
}

#' Model-Free Feature Screening Based on the Projection Correlation
#'
#' A model-free screening method is based on the projection correlation
#' which measures the dependence between two random vectors.
#' This projection correlation based method does not require specifying a
#' regression model, and applies to data in the presence of heavy tails
#' and multivariate responses. It enjoys both sure screening and
#' rank consistency properties under weak assumptions.
#'
#' @param X The design matrix of dimensions n * p. Each row is an observation vector.
#' @param Y The response vector of dimension n * 1.
#' @param nsis Number of predictors recruited by PCSIS. The default is n/log(n).
#' @return the labels of first nsis largest active set of all predictors
#'
#' @import reticulate
#' @importFrom reticulate py_config
#' @importFrom reticulate source_python
#' @import foreach
#' @import parallel
#' @import doParallel
#'
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#'
#'# have_numpy=reticulate::py_module_available("numpy")
#'# if (have_numpy){
#'#  req_py()
#' library(MFSIS)
#' n=100;
#' p=200;
#' rho=0.5;
#' data=GendataLM(n,p,rho,error="gaussian")
#' data=cbind(data[[1]],data[[2]])
#' colnames(data)[1:ncol(data)]=c(paste0("X",1:(ncol(data)-1)),"Y")
#' data=as.matrix(data)
#' X=data[,1:(ncol(data)-1)];
#' Y=data[,ncol(data)];
#'# A=PCSIS(X,Y,n/log(n));A
#'# }else{
#'#   print('You should have the Python testing environment!')
#'#}
#'
#' @references
#'
#' Zhu, L., K. Xu, R. Li, and W. Zhong (2017). Projection correlation between two random vectors. Biometrika 104(4), 829–843.
#'
#' Liu, W., Y. Ke, J. Liu, and R. Li (2020). Model-free feature screening and FDR control with knockoff features. Journal of the American Statistical Association, 1–16.
PCSIS=function(X,Y,nsis=(dim(X)[1])/log(dim(X)[1])){
  if (dim(X)[1]!=length(Y)) {
    stop("X and Y should have same number of rows!")
  }
  if (missing(X)|missing(Y)) {
    stop("The data is missing!")
  }
  if (TRUE%in%(is.na(X)|is.na(Y)|is.na(nsis))) {
    stop("The input vector or matrix cannot have NA!")
  }
  if (inherits(Y,"Surv")) {
    stop("PCSIS can not implemented with object  of Surv")
  }
  req_py()
  reticulate::py_config()
  py_path=system.file("python", "PCSIS.py", package="MFSIS")
  reticulate::source_python(py_path,envir=globalenv())
  n=dim(X)[1];
  p=dim(X)[2];
  A_y=get_arccos(Y)
  if (n*p<=100000){
    result=vector(mode="numeric",length=p)
    for (j in 1:p){
      result[j]=Cor(X[,j],A_y,n)
    }
  }else{
    # Real physical cores in the computer
    cores=detectCores(logical=FALSE)
    cl=makeCluster(cores)
    registerDoParallel(cl,cores=cores)
    j=NULL;
    result=foreach::foreach(j=1:p, .combine='c',
    .export=c("get_arccos","projection_corr"),
    .packages=c("reticulate")) %dopar%
      Cor(X[,j],A_y,n)
    stopImplicitCluster()
    stopCluster(cl)
  }
  A=order(result,decreasing=TRUE)
  return (A[1:nsis])
}




#' Arccos function
#'
#' This is a function to get a arccos value based on projection correlation from the Python language.
#'
#' @param X The design matrix of dimensions n * p. Each row is an observation vector.
#'
#' @import reticulate
#' @importFrom reticulate py_config
#' @importFrom reticulate source_python
#' @export
#' @return the arccos value
#'
#'
get_arccos=function(X){
  get_arccos_1d=NULL;
  py_path=system.file("python", "PCSIS.py", package="MFSIS")
  reticulate::source_python(py_path,envir=globalenv())
  A_x=get_arccos_1d(X);
  return (A_x)
}


#' Projection correlation function
#'
#' Projection correlation between X[,j] and Y from the Python language
#'
#' @param A_x  The arccos value about X
#' @param A_y  The arccos value about Y
#' @param n    The sample size
#' @import reticulate
#' @importFrom reticulate py_config
#' @importFrom reticulate source_python
#' @export
#' @return the projection correlation
#'
#'
projection_corr=function(A_x,A_y,n){
  projection_corr_1d=NULL;
  py_path=system.file("python", "PCSIS.py", package="MFSIS")
  reticulate::source_python(py_path,envir=globalenv())
  corr=projection_corr_1d(A_x,A_y,n);
  return(corr)
}


#' Parallel function
#' This is a parallel function about the projection correlation.
#'
#' @param Xj Each column from design matrix of dimensions n * p
#' @param A_y The arccos value about Y
#' @param n The sample size
#'
#' @return the projection correlation between Xj and A_y
#' @export
#'
Cor=function(Xj,A_y,n){
  A_x=get_arccos(Xj)
  corr=projection_corr(A_x,A_y,n);
  return (corr)
}

#' Generate simulation data (Complete data for intersection variables)
#'
#' This function helps you quickly generate simulation data based on transformation model.
#' You just need to input the sample and dimension of the data
#' you want to generate and the covariance parameter pho.
#' This simulated example comes from Section 4.2 introduced by Pan et al.(2019)
#'
#' @param n Number of subjects in the dataset to be simulated. It will also equal to the
#' number of rows in the dataset to be simulated, because it is assumed that each
#' row represents a different independent and identically distributed subject.
#' @param p Number of predictor variables (covariates) in the simulated dataset.
#' These covariates will be the features screened by model-free procedures.
#' @param rho The correlation between adjacent covariates in the simulated matrix X.
#' The within-subject covariance matrix of X is assumed to has the same form as an
#' AR(1) auto-regressive covariance matrix, although this is not meant to imply
#' that the X covariates for each subject are in fact a time series. Instead, it is just
#' used as an example of a parsimonious but nontrivial covariance structure. If
#' rho is left at the default of zero, the X covariates will be independent and the
#' simulation will run faster.
#' @param order The number of interactive variables and the default is 2.
#'
#' @return the list of your simulation data
#' @import MASS
#' @importFrom MASS mvrnorm
#' @importFrom stats rnorm
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataIM(n, p, rho)
#'
#' @references
#'
#' Pan, W., X. Wang, W. Xiao, and H. Zhu (2019). A generic sure independence screening procedure. Journal of the American Statistical Association 114(526), 928–937.
GendataIM <- function(n, p, rho,
                      order = 2) # n sample size; p dimension size.
{
  sig <- matrix(0, p, p)
  sig <- rho^abs(row(sig) - col(sig))
  diag(sig) <- rep(1, p)
  X <- mvrnorm(n, rep(0, p), sig)
  if (p < 21) {
    stop("The dimension p must be greater than 20")
  }
  if (order < 2 | order > 5) {
    stop("The parameter order can be implemented between 2 to 5")
  }
  if (order == 2) {
    Y <- 2 * X[, 1] * X[, 5] + 2 * X[, 10] + 2 * X[, 15] + 2 * X[, 20] + rnorm(n)
  } else if (order == 3) {
    Y <- 2 * X[, 1] * X[, 5] * X[, 10] + 2 * X[, 15] + 2 * X[, 20] + rnorm(n)
  } else if (order == 4) {
    Y <- 2 * X[, 1] * X[, 5] * X[, 10] * X[, 15] + 2 * X[, 20] + rnorm(n)
  } else if (order == 5) {
    Y <- 2 * X[, 1] * X[, 5] * X[, 10] * X[, 15] * X[, 20] + rnorm(n)
  }
  return(list(X = X, Y = Y))
}

#' @title Detect Python Module
#' @description  A function to detect Python module.
#' @name req_py
#' @aliases req_py
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @usage req_py()
#'
#' @import reticulate
#' @importFrom crayon cyan
#' @importFrom crayon blue
#' @import cli
#'
#' @export req_py
req_py <- function() {
  have_python <- reticulate::py_available()
  if (have_python) {
    message(paste0(crayon::cyan(cli::symbol$tick), crayon::blue("It is detected that you have configured the necessary Python environmemt! \n")))
  } else {
    message(paste0(crayon::cyan(cli::symbol$tick), crayon::blue("You should have the Python testing environment!\n")))
  }
  have_numpy <- reticulate::py_module_available("numpy")
  if (!have_numpy) {
    message(paste0(crayon::cyan(cli::symbol$tick), crayon::blue("Install the requirement Python module! \n")))
    reticulate::py_install("numpy")
  } else {
    message(paste0(crayon::cyan(cli::symbol$tick), crayon::blue("It is detected that you have configured the necessary Python module! \n")))
  }
}

#' Generate simulation data (Complete data with group predictors)
#'
#' In many regression problems, some predictors may be naturally grouped.
#' The most common example that contains group variables is the multifactor
#' analysis of variance (ANOVA) problem, where each factor may have several
#' levels and can be expressed through a group of dummy variables.
#' This function helps you quickly generate simulation data with group predictors.
#' You just need to input the sample and dimension of the data
#' you want to generate and the covariance parameter pho.
#' This simulated example comes from Example 2 introduced by Li et al.(2012)
#'
#' @param n Number of subjects in the dataset to be simulated. It will also equal to the
#' number of rows in the dataset to be simulated, because it is assumed that each
#' row represents a different independent and identically distributed subject.
#' @param p Number of predictor variables (covariates) in the simulated dataset.
#' These covariates will be the features screened by model-free procedures.
#' @param rho The correlation between adjacent covariates in the simulated matrix X.
#' The within-subject covariance matrix of X is assumed to has the same form as an
#' AR(1) auto-regressive covariance matrix, although this is not meant to imply
#' that the X covariates for each subject are in fact a time series. Instead, it is just
#' used as an example of a parsimonious but nontrivial covariance structure. If
#' rho is left at the default of zero, the X covariates will be independent and the
#' simulation will run faster.
#' @param error The distribution of error term.
#'
#' @return the list of your simulation data
#' @import MASS
#' @importFrom MASS mvrnorm
#' @importFrom stats rnorm
#' @importFrom stats rt
#' @importFrom stats rcauchy
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#' n <- 100
#' p <- 200
#' pho <- 0.5
#' data <- GendataGP(n, p, pho, "gaussian")
#'
#' @references
#'
#' Li, R., W. Zhong, and L. Zhu (2012). Feature screening via distance correlation learning. Journal of the American Statistical Association 107(499), 1129–1139.
GendataGP <- function(n, p, rho,
                      error = c("gaussian", "t", "cauchy")) # n sample size; p dimension size.
{
  sig <- matrix(0, p, p)
  sig <- rho^abs(row(sig) - col(sig))
  diag(sig) <- rep(1, p)
  X <- mvrnorm(n, rep(0, p), sig)
  q1 <- quantile(X[, 10], 0.25)
  q2 <- quantile(X[, 10], 0.5)
  q3 <- quantile(X[, 10], 0.75)
  y <- 2 * X[, 1] + 2 * X[, 5] + 2 * X[, 15] + 2 * X[, 20]
  +1 * as.numeric(X[, 10] < q1) + 2 * as.numeric(X[, 10] > q1 & X[, 10] <= q2)
  +3 * as.numeric(X[, 10] >= q3)
  if (error == "gaussian" | is.null(error)) {
    Y <- y + rnorm(n)
  } else if (error == "t") {
    Y <- y + rt(n, 2)
  } else if (error == "cauchy") {
    Y <- y + rcauchy(n)
  } else {
    stop("The author has not implemented this error term yet.")
  }
  return(list(X = X, Y = Y))
}

#' Generate simulation data (Survival data based on the Cox model)
#'
#' This function helps you quickly generate simulation data based on the Cox model.
#' You just need to input the sample and dimension of the data
#' you want to generate and the covariance parameter pho.
#'
#' @param n Number of subjects in the dataset to be simulated. It will also equal to the
#' number of rows in the dataset to be simulated, because it is assumed that each
#' row represents a different independent and identically distributed subject.
#' @param p Number of predictor variables (covariates) in the simulated dataset.
#' These covariates will be the features screened by model-free procedures.
#' @param rho The correlation between adjacent covariates in the simulated matrix X.
#' The within-subject covariance matrix of X is assumed to has the same form as an
#' AR(1) auto-regressive covariance matrix, although this is not meant to imply
#' that the X covariates for each subject are in fact a time series. Instead, it is just
#' used as an example of a parsimonious but nontrivial covariance structure. If
#' rho is left at the default of zero, the X covariates will be independent and the
#' simulation will run faster.
#' @param lambda This parameter control the censoring rate in survival data.
#' The censored time is generated by exponential distribution with mean 1/lambda. The default
#' is lambda=0.1.
#' @param beta A vector with length of n, which are the coefficients that you want to generate
#' about Linear model. The default is beta=(1,1,1,1,1,0,...,0)^T;
#'
#' @return the list of your simulation data
#' @import MASS
#' @importFrom MASS mvrnorm
#' @importFrom stats rexp
#' @importFrom stats runif
#'
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataCox(n, p, rho)
#'
#' @references
#' Cox DR (1972). “Regression models and life-tables.” Journal of the Royal Statistical Society:Series B (Methodological), 34(2), 187–202.
GendataCox <- function(n, p, rho,
                       beta = c(rep(1, 5), rep(0, p - 5)), lambda = 0.1) # n sample size; p dimension size.
{
  sig <- matrix(0, p, p)
  sig <- rho^abs(row(sig) - col(sig))
  diag(sig) <- rep(1, p)
  X <- mvrnorm(n, rep(0, p), sig)
  myrates <- diag(log(1 / runif(n))) %*% exp(-X %*% beta)
  Sur <- myrates
  CT <- rexp(n, lambda)
  time <- pmin(Sur, CT)
  status <- as.numeric(Sur <= CT)
  return(list(X = X, time = time, status = status))
}

#' Model-Free Feature Screening for Ultrahigh Dimensional Discriminant Analysis
#'
#' A marginal feature screening procedure based on empirical conditional distribution function.
#' The response variable is categorical in discriminant analysis.
#' This enables us to use the conditional distribution function to construct a new index for feature screening.
#'
#'
#' @param X The design matrix of dimensions n * p. Each row is an observation vector.
#' @param Y The response vector of dimension n * 1.
#' @param nsis Number of predictors recruited by MVSIS. The default is n/log(n).
#'
#' @return the labels of first nsis largest active set of all predictors
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#'
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataLGM(n, p, rho)
#' data <- cbind(data[[1]], data[[2]])
#' colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
#' data <- as.matrix(data)
#' X <- data[, 1:(ncol(data) - 1)]
#' Y <- data[, ncol(data)]
#' A <- MVSIS(X, Y, n / log(n))
#' A
#'
#' @references
#'
#' Cui, H., Li, R., & Zhong, W. (2015). Model-free feature screening for ultrahigh dimensional discriminant analysis. Journal of the American Statistical Association, 110(510), 630-641.
MVSIS <- function(X, Y, nsis) {
  if (dim(X)[1] != length(Y)) {
    stop("X and Y should have same number of rows!")
  }
  if (missing(X) | missing(Y)) {
    stop("The data is missing!")
  }
  if (TRUE %in% (is.na(X) | is.na(Y) | is.na(nsis))) {
    stop("The input vector or matrix cannot have NA!")
  }
  if (inherits(Y, "Surv")) {
    stop("SIRS can not implemented with object of Surv")
  }
  Fk <- function(X0, x) {
    # Returns the estimated value of F(X0) = P(X <= X0), specifically the proportion
    # of values in x that are less than a constant X0, where x is a numerical vector
    # representing the observed values of the kth covariate x.  This is the estimated
    # unconditional cumulative distribution function of X.
    Fk <- c()
    for (i in 1:length(x))
    {
      Fk[i] <- sum(X0 <= x[i]) / length(X0)
    }
    return(Fk)
  }
  Fkr <- function(X0, Y, yr, x) {
    # Returns the estimated value of Fr(X0) = P(X <= X0| Y=yr),
    # for variables (actually vectors) x and Y and for constants X0 and yr.
    # This is the estimated cumulative distribution function of X conditional
    # on the rth value of Y.
    Fkr <- c()
    ind_yr <- (Y == yr)
    for (i in 1:length(x))
    {
      Fkr[i] <- sum((X0 <= x[i]) * ind_yr) / sum(ind_yr)
    }
    return(Fkr)
  }
  MV <- function(X, Y) {
    # Calculates the MV nonparametric association measure between numerical variable X
    # and categorical variable Y, defined as expression (2.2) in Cui, H., Li, R., & Zhong, W. (2015).
    Fk0 <- Fk(X, X)
    # It is not a mistake that X is used twice here.  For each value of this covariate, we want
    # to know what quantile it represents.  The MV  is the Expectation in X of the variance in Y
    # of the conditional cumulative distribution function F(x|Y)=P(X<=x|Y).
    # Note that like the correlation or distance correlation, this is zero if X and Y are
    # independent because F(x) does not depend on Y in that case.
    Yr <- unique(Y)
    if (length(Yr) > 15) {
      stop("A supposedly categorical variable was provided with more than 15 levels.  Did you mean to treat it as numeric?")
    }
    MVr <- c()
    for (r in 1:length(Yr)) {
      MVr[r] <- (sum(Y == Yr[r]) / length(Y)) * mean((Fkr(X, Y, Yr[r], X) - Fk0)^2)
    }
    MV <- sum(MVr)
    return(MV)
  }
  MVy <- function(Xk) {
    MV(Xk, Y)
  }
  v <- abs(apply(X, 2, MVy))
  MWord <- order(v, decreasing = T) # rank order of estimated association strengths;
  A <- match(v, v[MWord])
  return(A[1:nsis])
}

#' Generate simulation data (The unified class framework to generate simulation data)
#'
#' This function helps you quickly generate simulation data.
#' You just need to input the sample and dimension of the data
#' you want to generate and the covariance parameter pho.
#' The models is numerous.
#'
#' @param n Number of subjects in the dataset to be simulated. It will also equal to the
#' number of rows in the dataset to be simulated, because it is assumed that each
#' row represents a different independent and identically distributed subject.
#' @param p Number of predictor variables (covariates) in the simulated dataset.
#' These covariates will be the features screened by model-free procedures.
#' @param rho The correlation between adjacent covariates in the simulated matrix X.
#' The within-subject covariance matrix of X is assumed to has the same form as an
#' AR(1) auto-regressive covariance matrix, although this is not meant to imply
#' that the X covariates for each subject are in fact a time series. Instead, it is just
#' used as an example of a parsimonious but nontrivial covariance structure. If
#' rho is left at the default of zero, the X covariates will be independent and the
#' simulation will run faster.
#' @param R A positive integer, number of outcome categories for multinomial (categorical) outcome Y.
#' @param beta A vector with length of n, which are the coefficients that you want to generate
#' about chosen model. The default is beta=(1,1,1,1,1,0,...,0)^T.
#' @param error The distribution of error term.
#' @param R A positive integer, number of outcome categories for multinomial (categorical) outcome Y.
#' @param lambda This parameter control the censoring rate in survival data.
#' The censored time is generated by exponential distribution with mean 1/lambda. The default
#' is lambda=0.1.
#' @param order The number of interactive variables and the default is 2.
#' @param style  Whether categories in categorial data are balanced or not.
#' @param type The type of multivariate response models, which use different mean and covariance
#' structure to generate data. Specially, type="a" is following the Model 3.a and
#' type="b" is following the Model 3.b by Liu et al.(2020).
#' @param model The model that you choose to generate simulation data.
#'
#' @return the list of your simulation data
#' @import MASS
#' @importFrom MASS mvrnorm
#' @importFrom stats rnorm
#' @importFrom stats rt
#' @importFrom stats rcauchy
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- Simdata(n, p, rho, error = "gaussian", model = "linear")
#' @references
#'
#' Liu, W., Y. Ke, J. Liu, and R. Li (2020). Model-free feature screening and FDR control with knockoff features. Journal of the American Statistical Association, 1–16.
Simdata <- function(n, p, rho,
                    beta = c(rep(1, 5), rep(0, p - 5)),
                    error = c("gaussian", "t", "cauchy"),
                    R = 3,
                    style = c("balanced", "unbalanced"),
                    lambda = 0.1,
                    order = 2,
                    type = c("a", "b"),
                    model = c(
                      "linear", "nonlinear", "binomial", "poisson", "classification",
                      "Cox", "interaction", "group", "multivariate", "AFT"
                    )) {
  if (model == "linear") {
    data <- GendataLM(n, p, rho, beta, error) ## NO.1
  } else if (model == "nonlinear") {
    data <- GendataTM(n, p, rho, beta, error) ## NO.2
  } else if (model == "binomial") {
    data <- GendataLGM(n, p, rho, beta) ## NO.3
  } else if (model == "poisson") {
    data <- GendataPM(n, p, rho, beta) ## NO.4
  } else if (model == "classification") {
    data <- GendataLDA(n, p, R, error, style) ## NO.5
  } else if (model == "Cox") {
    data <- GendataCox(n, p, rho, beta, lambda) ## NO.6
  } else if (model == "interaction") {
    data <- GendataIM(n, p, rho, order) ## NO.7
  } else if (model == "group") {
    data <- GendataGP(n, p, rho, error) ## NO.8
  } else if (model == "multivariate") {
    data <- GendataMRM(n, p, rho, type) ## NO.9
  } else if (model == "AFT") {
    data <- GendataAFT(n, p, rho, beta, lambda, error) ## NO.10
  } else {
    stop("The author has not implemented this model yet.")
  }
  return(data)
}

#' Generate simulation data (Categorial based on linear discriminant analysis model)
#'
#' @description Simulates a dataset that can be used to filter out features for ultrahigh-dimensional discriminant analysis.
#' The simulation is based on the balanced scenarios in Example 3.1 of Cui et al.(2015).
#' The simulated dataset has p numerical X-predictors and a categorical Y-response.
#'
#'
#' @param n Number of subjects in the dataset to be simulated. It will also equal to the
#' number of rows in the dataset to be simulated, because it is assumed that each
#' row represents a different independent and identically distributed subject.
#' @param p Number of predictor variables (covariates) in the simulated dataset.
#' These covariates will be the features screened by model-free procedures.
#' @param R A positive integer, number of outcome categories for multinomial (categorical) outcome Y.
#' @param error The distribution of error term, you can choose "gaussian" to generate a normal
#' distribution of error or you choose "t" to generate a t distribution of error with degree=2.
#' "cauchy" is represent the error term with cauchy distribution.
#' @param style The balance among categories in categorial data .
#'
#' @return the list of your simulation data
#' @importFrom stats rt
#' @importFrom stats rnorm
#' @importFrom stats rcauchy
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#' n <- 100
#' p <- 200
#' R <- 3
#' data <- GendataLDA(n, p, R, error = "gaussian", style = "balanced")
#' @references
#'
#' Cui, H., Li, R., & Zhong, W. (2015). Model-free feature screening for ultrahigh dimensional discriminant analysis. Journal of the American Statistical Association, 110(510), 630-641.
GendataLDA <- function(n, # number of subjects to be generated
                       p, # number of covariates to be generated
                       R = 3, # number of outcome categories for multinomial (categorical) outcome Y
                       error = c("gaussian", "t", "cauchy"), # The error distribution
                       style = c("balanced", "unbalanced")) { # Simulates multinomial Y for demonstrating MVSIS discriminant analysis.
  #  The p columns of X are each generated as independent;  most have
  #  means of zero, but the active covariates have different means depending on the level of Y.
  mu <- 3 # mu = signal strength
  if ((p < 30) | (p > 100000)) {
    stop("Please select a number p of predictors between 30 and 100000.")
  }
  if (style == "balanced") {
    Y <- sample(1:R, n, replace = T, prob = rep(1 / R, R))
  } else {
    P <- c()
    for (r in 1:R) {
      P[r] <- 2 * (1 + (r - 1) / (R - 1)) / (3 * R)
    }
    Y <- sample(1:R, n, replace = T, prob = P)
  }
  X <- matrix(0, n, p)
  for (r in 1:R) {
    ind <- (Y == r)
    A <- matrix(0, sum(ind), p)
    for (i in 1:sum(ind)) {
      # That is, for each subject having Y=r...
      if (error == "gaussian" | is.null(error)) {
        A[i, ] <- c(seq(0, 0, length = r - 1), mu, seq(0, 0, length = p - r)) + rnorm(p)
        # If heavyTailedCovariates = TRUE, then covariates will be generated as a mean
        # plus an independent error following a t distribution with 2 degrees of freedom.
      } else if (error == "t") {
        A[i, ] <- c(seq(0, 0, length = r - 1), mu, seq(0, 0, length = p - r)) + rt(p, 2)
        # If heavyTailedCovariates = TRUE, then covariates will be generated as a mean
        # plus a standard normal distribution, i.e., they will be independently normally
        # distributed with variance 1.
      } else if (error == "cauchy") {
        A[i, ] <- c(seq(0, 0, length = r - 1), mu, seq(0, 0, length = p - r)) + rcauchy(p)
      } else {
        stop("The author has not implemented this error term yet.")
      }
    }
    X[ind, ] <- A
  }
  #  The p columns of X are each generated as independent normals with variance one;  most have
  #  means of zero, but the active covariates have different means depending on the level of Y.
  #  Specifically, mu has been added to the rth predictor for r=1,...,R, so that the probability
  #  that Y equals r will be higher if the rth predictor is higher.  It is assumed that p>>r
  #  so that most predictors will be inactive.  In real data there is no reason why, say, the
  #  first two columns in the matrix should be the important ones, but this is convenient in a
  #  simulation and the choice of permutation of the columns involves no loss of generality.
  return(list(X = X, Y = Y))
}

#' Martingale Difference Correlation and Its Use in High-Dimensional Variable Screening
#'
#' A new metric, the so-called martingale difference correlation,
#' to measure the departure of conditional mean independence between a scalar response variable V and a vector predictor variable U.
#' Our metric is a natural extension of distance correlation proposed by Szekely, Rizzo, and Bahirov(2007),
#' which is used to measure the dependence between V and U. The martingale difference
#' correlation and its empirical counterpart inherit a number of desirable features of distance correlation and sample distance correlation,
#' such as algebraic simplicity and elegant theoretical properties.
#'
#' @param X The design matrix of dimensions n * p. Each row is an observation vector.
#' @param Y The response vector of dimension n * 1.
#' @param nsis Number of predictors recruited by MDCSIS. The default is n/log(n).
#'
#' @return the labels of first nsis largest active set of all predictors
#' @export
#' @author Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
#' @examples
#'
#' n <- 100
#' p <- 200
#' rho <- 0.5
#' data <- GendataLM(n, p, rho, error = "gaussian")
#' data <- cbind(data[[1]], data[[2]])
#' colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
#' data <- as.matrix(data)
#' X <- data[, 1:(ncol(data) - 1)]
#' Y <- data[, ncol(data)]
#' A <- MDCSIS(X, Y, n / log(n))
#' A
#'
#' @references
#'
#' Szekely, G. J., M. L. Rizzo, and N. K. Bakirov (2007). Measuring and testing dependence by correlation of distances. The annals of statistics 35(6), 2769–2794.
#'
#' Shao, X. and J. Zhang (2014). Martingale difference correlation and its use in high-dimensional variable screening. Journal of the American Statistical Association 109(507),1302–1318.
MDCSIS <- function(X, Y, nsis = (dim(X)[1]) / log(dim(X)[1])) {
  if (dim(X)[1] != length(Y)) {
    stop("X and Y should have same number of rows!")
  }
  if (missing(X) | missing(Y)) {
    stop("The data is missing!")
  }
  if (TRUE %in% (is.na(X) | is.na(Y) | is.na(nsis))) {
    stop("The input vector or matrix cannot have NA!")
  }
  if (inherits(Y, "Surv")) {
    stop("MDCSIS can not implemented with object  of Surv")
  }
  corr <- c()
  n <- dim(X)[1]
  p <- dim(X)[2]
  Y_ij <- Y %*% t(Y)
  for (k in 1:p) {
    X_matrix <- matrix(X[, k], n, n)
    X_ij <- abs(X_matrix - t(X_matrix))
    covxy <- mean(X_ij * Y_ij)
    covx <- mean(X_ij * X_ij)
    covy <- mean(Y_ij * Y_ij)
    corr[k] <- abs(covxy) / sqrt(covx * covy)
  }
  A <- order(corr, decreasing = TRUE)
  return(A[1:nsis])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PCSIS.R
\name{get_arccos}
\alias{get_arccos}
\title{Arccos function}
\usage{
get_arccos(X)
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}
}
\value{
the arccos value
}
\description{
This is a function to get a arccos value based on projection correlation from the Python language.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Kfilter.R
\name{Kfilter}
\alias{Kfilter}
\title{The Kolmogorov filter for variable screening}
\usage{
Kfilter(X, Y, nsis = (dim(X)[1])/log(dim(X)[1]))
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}

\item{Y}{The response vector of dimension n * 1.}

\item{nsis}{Number of predictors recruited by SIS. The default is n/log(n).}
}
\value{
the labels of first nsis largest active set of all predictors
}
\description{
A new model-free screening method called the fused Kolmogorov filter is proposed for high-dimensional data analysis.
This new method is fully nonparametric and can work with many types of covariates and response variables, including continuous,
discrete and categorical variables.
}
\examples{

n=100;
p=200;
rho=0.5;
data=GendataLM(n,p,rho,error="gaussian")
data=cbind(data[[1]],data[[2]])
colnames(data)[1:ncol(data)]=c(paste0("X",1:(ncol(data)-1)),"Y")
data=as.matrix(data)
X=data[,1:(ncol(data)-1)];
Y=data[,ncol(data)];
A=Kfilter(X,Y,n/log(n));A

}
\references{
Mai, Q., & Zou, H. (2013). The Kolmogorov filter for variable screening in high-dimensional binary classification. Biometrika, 100(1), 229-234.

Mai, Q., & Zou, H. (2015). The fused Kolmogorov filter: A nonparametric model-free screening method. The Annals of Statistics, 43(4), 1471-1497.
}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GendataLDA.R
\name{GendataLDA}
\alias{GendataLDA}
\title{Generate simulation data (Categorial based on linear discriminant analysis model)}
\usage{
GendataLDA(
  n,
  p,
  R = 3,
  error = c("gaussian", "t", "cauchy"),
  style = c("balanced", "unbalanced")
)
}
\arguments{
\item{n}{Number of subjects in the dataset to be simulated. It will also equal to the
number of rows in the dataset to be simulated, because it is assumed that each
row represents a different independent and identically distributed subject.}

\item{p}{Number of predictor variables (covariates) in the simulated dataset.
These covariates will be the features screened by model-free procedures.}

\item{R}{A positive integer, number of outcome categories for multinomial (categorical) outcome Y.}

\item{error}{The distribution of error term, you can choose "gaussian" to generate a normal
distribution of error or you choose "t" to generate a t distribution of error with degree=2.
"cauchy" is represent the error term with cauchy distribution.}

\item{style}{The balance among categories in categorial data .}
}
\value{
the list of your simulation data
}
\description{
Simulates a dataset that can be used to filter out features for ultrahigh-dimensional discriminant analysis.
The simulation is based on the balanced scenarios in Example 3.1 of Cui et al.(2015).
The simulated dataset has p numerical X-predictors and a categorical Y-response.
}
\examples{
n <- 100
p <- 200
R <- 3
data <- GendataLDA(n, p, R, error = "gaussian", style = "balanced")
}
\references{
Cui, H., Li, R., & Zhong, W. (2015). Model-free feature screening for ultrahigh dimensional discriminant analysis. Journal of the American Statistical Association, 110(510), 630-641.
}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MFSIS.R
\name{MFSIS}
\alias{MFSIS}
\title{Model-free feature screening procedures}
\usage{
MFSIS(
  X,
  Y,
  nsis = (dim(X)[1])/log(dim(X)[1]),
  method = c("SIS", "SIRS", "DCSIS", "MDCSIS", "CSIS", "PCSIS", "BcorSIS", "WLS",
    "MVSIS", "Kfilter")
)
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}

\item{Y}{The response vector of dimension n * 1.}

\item{nsis}{Number of predictors recruited by the screening method. The default is n/log(n).}

\item{method}{The method that you choose to perform screening procedure.
method=c("SIS", "SIRS", "DCSIS", "MDCSIS", "CSIS", "PCSIS", "BcorSIS", "WLS", "MVSIS", "Kfilter","PSIS","CAS","CISIS").
If you want to know more information about this method, please use command "help(method)" for detail information.}
}
\value{
the labels of first nsis largest active set of all predictors
}
\description{
Through this function, we provide a unified framework
to carry out model-free screening procedures including
SIS (Fan and Lv (2008) <doi:10.1111/j.1467-9868.2008.00674.x>),
SIRS(Zhu et al. (2011)<doi:10.1198/jasa.2011.tm10563>),
 DC-SIS (Li et al. (2012) <doi:10.1080/01621459.2012.695654>),
MDC-SIS(Shao and Zhang (2014) <doi:10.1080/01621459.2014.887012>),
Bcor-SIS (Pan et al. (2019) <doi:10.1080/01621459.2018.1462709>),
 PC-Screen (Liu et al. (2020) <doi:10.1080/01621459.2020.1783274>),
WLS (Zhong et al.(2021) <doi:10.1080/01621459.2021.1918554>),
Kfilter (Mai and Zou (2015) <doi:10.1214/14-AOS1303>),
MVSIS (Cui et al. (2015) <doi:10.1080/01621459.2014.920256>),
PSIS (Pan et al. (2016) <doi:10.1080/01621459.2014.998760>),
 CAS (Xie et al. (2020) <doi:101080/0162145920191573734>),
CI-SIS (Cheng and Wang. (2022) <doi:10.1016/j.cmpb.2022.107269>)
and CSIS.
}
\examples{

n <- 100
p <- 200
rho <- 0.5
data <- GendataLM(n, p, rho, error = "gaussian")
data <- cbind(data[[1]], data[[2]])
colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
data <- as.matrix(data)
X <- data[, 1:(ncol(data) - 1)]
Y <- data[, ncol(data)]
A <- MFSIS(X, Y, n / log(n), method = "CSIS")
A

}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MVSIS.R
\name{MVSIS}
\alias{MVSIS}
\title{Model-Free Feature Screening for Ultrahigh Dimensional Discriminant Analysis}
\usage{
MVSIS(X, Y, nsis)
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}

\item{Y}{The response vector of dimension n * 1.}

\item{nsis}{Number of predictors recruited by MVSIS. The default is n/log(n).}
}
\value{
the labels of first nsis largest active set of all predictors
}
\description{
A marginal feature screening procedure based on empirical conditional distribution function.
The response variable is categorical in discriminant analysis.
This enables us to use the conditional distribution function to construct a new index for feature screening.
}
\examples{

n <- 100
p <- 200
rho <- 0.5
data <- GendataLGM(n, p, rho)
data <- cbind(data[[1]], data[[2]])
colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
data <- as.matrix(data)
X <- data[, 1:(ncol(data) - 1)]
Y <- data[, ncol(data)]
A <- MVSIS(X, Y, n / log(n))
A

}
\references{
Cui, H., Li, R., & Zhong, W. (2015). Model-free feature screening for ultrahigh dimensional discriminant analysis. Journal of the American Statistical Association, 110(510), 630-641.
}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LogrankSIS.R
\name{logrank_score}
\alias{logrank_score}
\title{Variance-adjusted generalized log-rank utility for one binary feature}
\usage{
logrank_score(x_k, g)
}
\arguments{
\item{x_k}{Integer vector of length n with values in \{0, 1\}.}

\item{g}{List returned by \code{logrank_globals()}.}
}
\value{
A single non-negative numeric value.
}
\description{
Computes \eqn{\hat\omega_k = |\hat U_k| / [n \hat\pi_k (1 - \hat\pi_k)]}
using precomputed global quantities from \code{logrank_globals()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GendataIC.R
\name{GendataIC}
\alias{GendataIC}
\title{Generate Simulation Data for Interval-Censored Screening}
\usage{
GendataIC(
  n,
  p,
  rho,
  model = c("linear", "poisson", "cox"),
  balanced = TRUE,
  censoring = 1L,
  seed = NULL
)
}
\arguments{
\item{n}{Number of subjects (rows of the design matrix).}

\item{p}{Number of predictors (columns of the design matrix).}

\item{rho}{AR(1) correlation between adjacent latent predictors.
Must satisfy \eqn{0 \le \rho < 1}.}

\item{model}{Character string specifying the model for the latent response.
One of \code{"linear"}, \code{"poisson"}, or \code{"cox"}.
\describe{
  \item{\code{"linear"}}{Linear model \eqn{Y^* = \mathbf{x}^\top\boldsymbol\beta + \epsilon},
    \eqn{\epsilon \sim N(0,1)}.}
  \item{\code{"poisson"}}{Poisson regression with
    \eqn{\log\lambda = \mathbf{x}^\top\boldsymbol\beta},
    \eqn{\boldsymbol\beta = (1, 0.8, 0.6, \mathbf{0}_{15}^\top, -0.6, -0.8,
    \mathbf{0}_{p-20}^\top)^\top}.}
  \item{\code{"cox"}}{Cox proportional hazards with Weibull baseline
    (shape = 2, scale = 0.15) and
    \eqn{\boldsymbol\beta = (1.2, 1, 0.8, \mathbf{0}_{15}^\top, -1, -1.2,
    \mathbf{0}_{p-20}^\top)^\top}.}
}}

\item{balanced}{Logical.  If \code{TRUE} (default), predictors are
dichotomized at the median (\eqn{X_k = I(X^*_k > 0)}), yielding
approximately equal group sizes.  If \code{FALSE}, the 70th percentile
of \eqn{N(0,1)} is used as the threshold, producing imbalanced
(sparse) binary predictors.}

\item{censoring}{Integer 1 or 2 selecting the censoring scenario:
\describe{
  \item{\code{1}}{(default) \eqn{(p_I, p_R, p_L) = (80\%, 14\%, 6\%)}.}
  \item{\code{2}}{\eqn{(p_I, p_R, p_L) = (80\%, 6\%, 14\%)}.}
}}

\item{seed}{Optional integer seed passed to \code{set.seed()} for
reproducibility.  \code{NULL} (default) leaves the RNG state unchanged.}
}
\value{
A list with three elements:
  \describe{
    \item{\code{X}}{An \eqn{n \times p} binary integer matrix of predictors.}
    \item{\code{L}}{Numeric vector of length \eqn{n}: lower interval bounds.
      Left-censored observations have \code{L = -Inf}.}
    \item{\code{R}}{Numeric vector of length \eqn{n}: upper interval bounds.
      Right-censored observations have \code{R = Inf}.}
  }
}
\description{
Generates ultrahigh-dimensional simulation data for evaluating feature
screening methods under Case II interval censoring, following the
data-generating framework of the Log-rank SIS paper.
The true active predictor set is fixed at \eqn{\mathcal{D} = \{1,2,3,19,20\}}.

Binary predictors are obtained by dichotomizing a latent Gaussian vector
with Toeplitz covariance \eqn{\Sigma_{jk} = \rho^{|j-k|}}.  The latent
response \eqn{Y^*} is generated from one of three models (linear, Poisson,
or Cox), and interval-censored observations \eqn{(Y^L_i, Y^U_i]} are
produced by adding asymmetric random noise to \eqn{Y^*_i}.
}
\examples{
## Linear model, balanced predictors, censoring scenario 1
data <- GendataIC(n = 100, p = 500, rho = 0.5,
                  model = "linear", balanced = TRUE,
                  censoring = 1, seed = 2025)
X <- data$X; L <- data$L; R <- data$R
cat("Dimensions: n =", nrow(X), "  p =", ncol(X), "\n")
cat("Right-censored:", sum(is.infinite(R)), "\n")
cat("Left-censored: ", sum(is.infinite(L)), "\n")

## Poisson model, imbalanced predictors, censoring scenario 2
data2 <- GendataIC(n = 100, p = 500, rho = 0.2,
                   model = "poisson", balanced = FALSE,
                   censoring = 2, seed = 42)

## Cox model
data3 <- GendataIC(n = 100, p = 500, rho = 0.5,
                   model = "cox", balanced = TRUE, seed = 1)

}
\references{
Fan, J. and Lv, J. (2008). Sure independence screening for ultrahigh
dimensional feature space.
\emph{Journal of the Royal Statistical Society: Series B},
\strong{70}(5), 849--911.

Zhong, W., Qian, C., Liu, W., Zhu, L., and Li, R. (2023). Feature
screening for interval-valued response with application to study
association between posted salary and required skills.
\emph{Journal of the American Statistical Association},
\strong{118}, 805--817.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LogrankSIS.R
\name{LogrankSIS}
\alias{LogrankSIS}
\title{Generalized Log-rank Sure Independence Screening for Ultrahigh-Dimensional
Interval-Censored Response}
\usage{
LogrankSIS(X, L, R, nsis = floor(dim(X)[1]/log(dim(X)[1])))
}
\arguments{
\item{X}{The design matrix of dimensions \eqn{n \times p}.  Each row is an
observation vector.  Predictors should be binary (0/1), as commonly
arising from SNP minor-allele encoding or word-segmentation of job
postings.}

\item{L}{A numeric vector of length \eqn{n} giving the \strong{lower}
endpoints of the observed intervals \eqn{(L_i, R_i]}.  For left-censored
observations (event known to have occurred no later than \eqn{R_i}), set
\code{L[i] = 0} or \code{L[i] = -Inf} as appropriate for the application.}

\item{R}{A numeric vector of length \eqn{n} giving the \strong{upper}
endpoints of the observed intervals.  Set \code{R[i] = Inf} for
right-censored observations (event not yet observed by the end of
follow-up).}

\item{nsis}{Number of predictors recruited by \code{LogrankSIS}.  The
default is \code{floor(n / log(n))}.}
}
\value{
An integer vector of length \code{nsis} containing the
  \strong{column indices} of the top-ranked predictors in \code{X},
  ordered by decreasing marginal utility \eqn{\hat\omega_k}.
}
\description{
A computationally efficient and statistically robust feature screening
procedure for ultrahigh-dimensional Case II interval-censored responses
with binary predictors (e.g., SNP indicators or text-derived skill words).

The marginal utility is the \emph{variance-adjusted generalized log-rank
statistic}
\deqn{\hat\omega_k = \frac{|\hat U_k|}{n\,\hat\pi_k(1-\hat\pi_k)},}
where \eqn{\hat U_k} is the generalized log-rank score of Sun (1996)
evaluated at the pooled NPMLE \eqn{\hat{\mathbf p}} fitted once to the
full sample under the null hypothesis of no covariate effect.  Because
\eqn{\hat{\mathbf p}} and the resulting posterior weights \eqn{w_{ij}} are
shared across all \eqn{p} features, evaluating each feature reduces to
simple vector operations, achieving dramatic speed-ups over ADD-SIS
(Zhong et al., 2023), which requires \eqn{2p} stratified NPMLE fits.
}
\examples{
## ---- Example 1: Balanced binary predictors, linear model ----
set.seed(2025)
n   <- 200
p   <- 1000
rho <- 0.5

## Toeplitz covariance for latent continuous predictors
data <- GendataIC(n, p, rho, model = "linear", balanced = TRUE)
X <- data[[1]]   # n x p binary design matrix
L <- data[[2]]   # lower bounds
R <- data[[3]]   # upper bounds (Inf = right-censored)

## Run LogrankSIS (active predictors are indices 1, 2, 3, 19, 20)
result <- LogrankSIS(X, L, R, nsis = floor(n / log(n)))
cat("Active predictors recovered:", intersect(result, c(1, 2, 3, 19, 20)), "\n")

## ---- Example 2: Imbalanced binary predictors, Cox model ----
set.seed(2025)
data2 <- GendataIC(n, p, rho, model = "cox", balanced = FALSE)
X2 <- data2[[1]]; L2 <- data2[[2]]; R2 <- data2[[3]]

result2 <- LogrankSIS(X2, L2, R2, nsis = floor(n / log(n)))
cat("Active predictors recovered:", intersect(result2, c(1, 2, 3, 19, 20)), "\n")

}
\references{
Sun, J. (1996). A non-parametric test for interval-censored failure time
data with application to AIDS studies.
\emph{Statistics in Medicine}, \strong{15}, 1387--1395.

Zhao, Q. and Sun, J. (2004). Generalized log-rank test for mixed
interval-censored failure time data.
\emph{Statistics in Medicine}, \strong{23}(10), 1621--1629.

Zhong, W., Qian, C., Liu, W., Zhu, L., and Li, R. (2023). Feature
screening for interval-valued response with application to study
association between posted salary and required skills.
\emph{Journal of the American Statistical Association},
\strong{118}, 805--817.

Fan, J. and Lv, J. (2008). Sure independence screening for ultrahigh
dimensional feature space.
\emph{Journal of the Royal Statistical Society: Series B},
\strong{70}(5), 849--911.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/WLS.R
\name{WLS}
\alias{WLS}
\title{A Model-free Variable Screening Method Based on Leverage Score}
\usage{
WLS(X, Y, nsis = (dim(X)[1])/log(dim(X)[1]))
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}

\item{Y}{The response vector of dimension n * 1.}

\item{nsis}{Number of predictors recruited by WLS. The default is n/log(n).}
}
\value{
the labels of first nsis largest active set of all predictors.
}
\description{
An innovative and effective sampling scheme based on leverage scores via singular value decompositions
has been proposed to select rows of a design matrix as a surrogate of the full data in linear regression.
Analogously, variable screening can be viewed as selecting rows of the design matrix. However, effective
variable selection along this line of thinking remains elusive. This method propose a
weighted leverage variable screening method by using both the left and right singular vectors of the design matrix.
}
\examples{

n <- 100
p <- 200
rho <- 0.5
data <- GendataLM(n, p, rho, error = "gaussian")
data <- cbind(data[[1]], data[[2]])
colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
data <- as.matrix(data)
X <- data[, 1:(ncol(data) - 1)]
Y <- data[, ncol(data)]
A <- WLS(X, Y, n / log(n))
A

}
\references{
Zhong, W., Liu, Y., & Zeng, P. (2021). A Model-free Variable Screening Method Based on Leverage Score. Journal of the American Statistical Association, (just-accepted), 1-36.
}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PCSIS.R
\name{projection_corr}
\alias{projection_corr}
\title{Projection correlation function}
\usage{
projection_corr(A_x, A_y, n)
}
\arguments{
\item{A_x}{The arccos value about X}

\item{A_y}{The arccos value about Y}

\item{n}{The sample size}
}
\value{
the projection correlation
}
\description{
Projection correlation between X[,j] and Y from the Python language
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LogrankSIS.R
\name{logrank_globals}
\alias{logrank_globals}
\title{Precompute global log-rank quantities shared across all p features}
\usage{
logrank_globals(l, r)
}
\arguments{
\item{l}{Numeric vector of lower bounds.}

\item{r}{Numeric vector of upper bounds.}
}
\value{
A list with components \code{w}, \code{d_j}, \code{n_j}, \code{n}.
}
\description{
Fits a single pooled NPMLE under the null (no covariate effect) and
computes posterior weights \eqn{w_{ij}}, pooled pseudo-failure counts
\eqn{\hat{d}_j}, and pooled pseudo-risk-set sizes \eqn{\hat{n}_j}.
These are reused for every feature at negligible additional cost.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GendataCox.R
\name{GendataCox}
\alias{GendataCox}
\title{Generate simulation data (Survival data based on the Cox model)}
\usage{
GendataCox(n, p, rho, beta = c(rep(1, 5), rep(0, p - 5)), lambda = 0.1)
}
\arguments{
\item{n}{Number of subjects in the dataset to be simulated. It will also equal to the
number of rows in the dataset to be simulated, because it is assumed that each
row represents a different independent and identically distributed subject.}

\item{p}{Number of predictor variables (covariates) in the simulated dataset.
These covariates will be the features screened by model-free procedures.}

\item{rho}{The correlation between adjacent covariates in the simulated matrix X.
The within-subject covariance matrix of X is assumed to has the same form as an
AR(1) auto-regressive covariance matrix, although this is not meant to imply
that the X covariates for each subject are in fact a time series. Instead, it is just
used as an example of a parsimonious but nontrivial covariance structure. If
rho is left at the default of zero, the X covariates will be independent and the
simulation will run faster.}

\item{beta}{A vector with length of n, which are the coefficients that you want to generate
about Linear model. The default is beta=(1,1,1,1,1,0,...,0)^T;}

\item{lambda}{This parameter control the censoring rate in survival data.
The censored time is generated by exponential distribution with mean 1/lambda. The default
is lambda=0.1.}
}
\value{
the list of your simulation data
}
\description{
This function helps you quickly generate simulation data based on the Cox model.
You just need to input the sample and dimension of the data
you want to generate and the covariance parameter pho.
}
\examples{
n <- 100
p <- 200
rho <- 0.5
data <- GendataCox(n, p, rho)

}
\references{
Cox DR (1972). “Regression models and life-tables.” Journal of the Royal Statistical Society:Series B (Methodological), 34(2), 187–202.
}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LogrankSIS.R
\name{turnbull_em}
\alias{turnbull_em}
\title{Turnbull EM algorithm for NPMLE of interval-censored data}
\usage{
turnbull_em(l, r, supp, max_iter = 1000, tol = 1e-10)
}
\arguments{
\item{l}{Numeric vector of lower bounds.}

\item{r}{Numeric vector of upper bounds (Inf for right-censored).}

\item{supp}{Numeric vector of support points.}

\item{max_iter}{Maximum number of EM iterations.}

\item{tol}{Convergence tolerance.}
}
\value{
Numeric vector of probability masses on \code{supp}.
}
\description{
Turnbull EM algorithm for NPMLE of interval-censored data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BcorSIS.R
\name{BcorSIS}
\alias{BcorSIS}
\title{A Generic Sure Independence Screening Procedure}
\usage{
BcorSIS(X, Y, nsis = (dim(X)[1])/log(dim(X)[1]))
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}

\item{Y}{The response vector of dimension n * 1. For survival models, Y should
be an object of class Surv, as provided by the function
Surv() in the package survival.}

\item{nsis}{Number of predictors recruited by BcorSIS. The default is n/log(n).}
}
\value{
the labels of first nsis largest active set of all predictors
}
\description{
A generic nonparametric sure independence screening procedure,
called BCor-SIS, on the basis of a recently developed universal
dependence measure: Ball correlation.
We show that the proposed procedure has strong screening consistency even
when the dimensionality is an exponential order of the sample size
without imposing sub-exponential moment assumptions on the data.
}
\examples{

## Scenario 1  generate complete data
n <- 100
p <- 200
rho <- 0.5
data <- GendataLM(n, p, rho, error = "gaussian")
data <- cbind(data[[1]], data[[2]])
colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
data <- as.matrix(data)
X <- data[, 1:(ncol(data) - 1)]
Y <- data[, ncol(data)]
A1 <- BcorSIS(X, Y, n / log(n))
A1

## Scenario 2  generate survival data
library(survival)
n <- 100
p <- 200
rho <- 0.5
data <- GendataCox(n, p, rho)
data <- cbind(data[[1]], data[[2]], data[[3]])
colnames(data)[ncol(data)] <- c("status")
colnames(data)[(ncol(data) - 1)] <- c("time")
colnames(data)[(1:(ncol(data) - 2))] <- c(paste0("X", 1:(ncol(data) - 2)))
data <- as.matrix(data)
X <- data[, 1:(ncol(data) - 2)]
Y <- Surv(data[, (ncol(data) - 1)], data[, ncol(data)])
A2 <- BcorSIS(X, Y, n / log(n))
A2
}
\references{
Pan, W., X. Wang, H. Zhang, H. Zhu, and J. Zhu (2020). Ball covariance: A generic measure of dependence in banach space. Journal of the American Statistical Association 115(529),307–317.

Pan, W., X. Wang, W. Xiao, and H. Zhu (2019). A generic sure independence screening procedure. Journal of the American Statistical Association 114(526), 928–937.
}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Kfilter.R
\name{Kfilter_fused}
\alias{Kfilter_fused}
\title{The fused kolmogorov filter: a nonparametric model-free screening method}
\usage{
Kfilter_fused(X, Y, nsis = (dim(X)[1])/log(dim(X)[1]))
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}

\item{Y}{The response vector of dimension n * 1.}

\item{nsis}{Number of predictors recruited by Kfilter_fused. The default is n/log(n).}
}
\value{
the labels of first nsis largest active set of all predictors
}
\description{
The fused kolmogorov filter: a nonparametric model-free screening method
}
\examples{
##Scenario 1  generate discrete response data
n=100;
p=200;
R=5;
data=GendataLDA(n,p,R,error="gaussian",style="balanced")
data=cbind(data[[1]],data[[2]])
colnames(data)[1:ncol(data)]=c(paste0("X",1:(ncol(data)-1)),"Y")
data=as.matrix(data)
X=data[,1:(ncol(data)-1)];
Y=data[,ncol(data)];
A1=Kfilter_fused(X,Y,n/log(n));A1

##Scenario 2  generate continuous response data
n=50;
p=200;
rho=0.5;
data=GendataLM(n,p,rho,error="gaussian")
data=cbind(data[[1]],data[[2]])
colnames(data)[1:ncol(data)]=c(paste0("X",1:(ncol(data)-1)),"Y")
data=as.matrix(data)
X=data[,1:(ncol(data)-1)];
Y=data[,ncol(data)];
A2=Kfilter_fused(X,Y,n/log(n));A2

}
\references{
Mai, Q., & Zou, H. (2015). The fused Kolmogorov filter: A nonparametric model-free screening method. The Annals of Statistics, 43(4), 1471-1497.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Kfilter.R
\name{Kfilter_single}
\alias{Kfilter_single}
\title{The Kolmogorov filter for variable screening in high-dimensional binary classification}
\usage{
Kfilter_single(X, Y, nsis = (dim(X)[1])/log(dim(X)[1]))
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}

\item{Y}{The response vector of dimension n * 1.}

\item{nsis}{Number of predictors recruited by Kfilter_single. The default is n/log(n).}
}
\value{
the labels of first nsis largest active set of all predictors
}
\description{
The Kolmogorov filter for variable screening in high-dimensional binary classification
}
\examples{
n=100;
p=200;
rho=0.5;
data=GendataLGM(n,p,rho)
data=cbind(data[[1]],data[[2]])
colnames(data)[1:ncol(data)]=c(paste0("X",1:(ncol(data)-1)),"Y")
data=as.matrix(data)
X=data[,1:(ncol(data)-1)];
Y=data[,ncol(data)];
A=Kfilter_single(X,Y,n/log(n));A

}
\references{
#' Mai, Q., & Zou, H. (2013). The Kolmogorov filter for variable screening in high-dimensional binary classification. Biometrika, 100(1), 229-234.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/req_py.R
\name{req_py}
\alias{req_py}
\title{Detect Python Module}
\usage{
req_py()
}
\description{
A function to detect Python module.
}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PSIS.R
\name{PSIS}
\alias{PSIS}
\title{Ultrahigh-Dimensional Multiclass Linear Discriminant Analysis by Pairwise Sure Independence Screening}
\usage{
PSIS(X, Y, nsis)
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}

\item{Y}{The response vector of dimension n * 1.}

\item{nsis}{Number of predictors recruited by PSIS. The default is n/log(n).}
}
\value{
the labels of first nsis largest active set of all predictors
}
\description{
A novel pairwise sure independence screening method for linear discriminant analysis with an
ultrahigh-dimensional predictor. This procedure is directly applicable to the situation with many classes.
}
\examples{

n <- 100
p <- 200
rho <- 0.5
data <- GendataLGM(n, p, rho)
data <- cbind(data[[1]], data[[2]])
colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
data <- as.matrix(data)
X <- data[, 1:(ncol(data) - 1)]
Y <- data[, ncol(data)]
A <- PSIS(X, Y, n / log(n))
A

}
\references{
Pan, R., Wang, H., and Li, R. (2016). Ultrahigh-dimensional multiclass linear discriminant analysis by pairwise sure independence screening. Journal of the American Statistical Association, 111(513):169–179.
}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CISIS.R
\name{CISIS}
\alias{CISIS}
\title{Model-Free Feature screening Based on Concordance Index for Ultra-High Dimensional Categorical Data}
\usage{
CISIS(X, Y, nsis)
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}

\item{Y}{The response vector of dimension n * 1.}

\item{nsis}{Number of predictors recruited by CISIS. The default is n/log(n).}
}
\value{
the labels of first nsis largest active set of all predictors
}
\description{
The proposed method is based on the concordance index which measures concordance between random vectors.
A model-free and robust feature screening method for ultrahigh-dimensional categorical data.
The performance is quite robust in the presence of heavy-tailed distributions, extremely unbalance responses, and category-adaptive data.
}
\examples{

n <- 100
p <- 200
rho <- 0.5
data <- GendataLGM(n, p, rho)
data <- cbind(data[[1]], data[[2]])
colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
data <- as.matrix(data)
X <- data[, 1:(ncol(data) - 1)]
Y <- data[, ncol(data)]
A <- CISIS(X, Y, n / log(n))
A

}
\references{
Cheng X, Wang H. A generic model-free feature screening procedure for ultra-high dimensional data with categorical response[J]. Computer Methods and Programs in Biomedicine, 2023, 229: 107269.
}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PCSIS.R
\name{Cor}
\alias{Cor}
\title{Parallel function
This is a parallel function about the projection correlation.}
\usage{
Cor(Xj, A_y, n)
}
\arguments{
\item{Xj}{Each column from design matrix of dimensions n * p}

\item{A_y}{The arccos value about Y}

\item{n}{The sample size}
}
\value{
the projection correlation between Xj and A_y
}
\description{
Parallel function
This is a parallel function about the projection correlation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MDCSIS.R
\name{MDCSIS}
\alias{MDCSIS}
\title{Martingale Difference Correlation and Its Use in High-Dimensional Variable Screening}
\usage{
MDCSIS(X, Y, nsis = (dim(X)[1])/log(dim(X)[1]))
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}

\item{Y}{The response vector of dimension n * 1.}

\item{nsis}{Number of predictors recruited by MDCSIS. The default is n/log(n).}
}
\value{
the labels of first nsis largest active set of all predictors
}
\description{
A new metric, the so-called martingale difference correlation,
to measure the departure of conditional mean independence between a scalar response variable V and a vector predictor variable U.
Our metric is a natural extension of distance correlation proposed by Szekely, Rizzo, and Bahirov(2007),
which is used to measure the dependence between V and U. The martingale difference
correlation and its empirical counterpart inherit a number of desirable features of distance correlation and sample distance correlation,
such as algebraic simplicity and elegant theoretical properties.
}
\examples{

n <- 100
p <- 200
rho <- 0.5
data <- GendataLM(n, p, rho, error = "gaussian")
data <- cbind(data[[1]], data[[2]])
colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
data <- as.matrix(data)
X <- data[, 1:(ncol(data) - 1)]
Y <- data[, ncol(data)]
A <- MDCSIS(X, Y, n / log(n))
A

}
\references{
Szekely, G. J., M. L. Rizzo, and N. K. Bakirov (2007). Measuring and testing dependence by correlation of distances. The annals of statistics 35(6), 2769–2794.

Shao, X. and J. Zhang (2014). Martingale difference correlation and its use in high-dimensional variable screening. Journal of the American Statistical Association 109(507),1302–1318.
}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SIRS.R
\name{SIRS}
\alias{SIRS}
\title{Model-Free Feature Screening for Ultrahigh Dimensional Data}
\usage{
SIRS(X, Y, nsis = (dim(X)[1])/log(dim(X)[1]))
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}

\item{Y}{The response vector of dimension n * 1.}

\item{nsis}{Number of predictors recruited by SIRS. The default is n/log(n).}
}
\value{
the labels of first nsis largest active set of all predictors
}
\description{
A novel feature screening procedure under a unified model framework,
which covers a wide variety of commonly used parametric and semi-parametric models.
This method does not require imposing a specific model structure on regression functions,
and thus is particularly appealing to ultrahigh-dimensional regressions, where there are a
huge number of candidate predictors but little information about the actual model forms.
}
\examples{

n <- 100
p <- 200
rho <- 0.5
data <- GendataLM(n, p, rho, error = "gaussian")
data <- cbind(data[[1]], data[[2]])
colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
data <- as.matrix(data)
X <- data[, 1:(ncol(data) - 1)]
Y <- data[, ncol(data)]
A <- SIRS(X, Y, n / log(n))
A

}
\references{
Zhu, L.-P., L. Li, R. Li, and L.-X. Zhu (2011). Model-free feature screening for ultrahigh-dimensional data. Journal of the American Statistical Association 106(496), 1464–1475.
}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CSIS.R
\name{CSIS}
\alias{CSIS}
\title{Model-Free Feature screening Based on Concordance Index Statistic}
\usage{
CSIS(X, Y, nsis = (dim(X)[1])/log(dim(X)[1]))
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}

\item{Y}{The response vector of dimension n * 1. For survival models,
Y should be an object of class Surv, as provided by the function
Surv() in the package survival.}

\item{nsis}{Number of predictors recruited by CSIS. The default is n/log(n).}
}
\value{
the labels of first nsis largest active set of all predictors
}
\description{
A model-free and data-adaptive feature screening method for
ultrahigh-dimensional data and even survival data. The proposed method is based
on the concordance index which measures concordance between random vectors even
if one of the vectors is a survival object Surv. This rank correlation based
method does not require specifying a regression model, and applies robustly to data
in the presence of censoring and heavy tails. It enjoys both sure screening and rank
consistency properties under weak assumptions.
}
\examples{

## Scenario 1  generate complete data
n <- 100
p <- 200
rho <- 0.5
data <- GendataLM(n, p, rho, error = "gaussian")
data <- cbind(data[[1]], data[[2]])
colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
data <- as.matrix(data)
X <- data[, 1:(ncol(data) - 1)]
Y <- data[, ncol(data)]
A1 <- CSIS(X, Y, n / log(n))
A1

## Scenario 2  generate survival data
library(survival)
n <- 100
p <- 200
rho <- 0.5
data <- GendataCox(n, p, rho)
data <- cbind(data[[1]], data[[2]], data[[3]])
colnames(data)[ncol(data)] <- c("status")
colnames(data)[(ncol(data) - 1)] <- c("time")
colnames(data)[(1:(ncol(data) - 2))] <- c(paste0("X", 1:(ncol(data) - 2)))
data <- as.matrix(data)
X <- data[, 1:(ncol(data) - 2)]
Y <- Surv(data[, (ncol(data) - 1)], data[, ncol(data)])
A2 <- CSIS(X, Y, n / log(n))
A2

}
\references{
Cheng X, Li G, Wang H. The concordance filter: an adaptive model-free feature screening procedure[J]. Computational Statistics, 2024, 39:2413–2436.
}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SIS.R
\name{SIS}
\alias{SIS}
\title{Sure Independent Screening}
\usage{
SIS(X, Y, nsis = (dim(X)[1])/log(dim(X)[1]))
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}

\item{Y}{The response vector of dimension n * 1.}

\item{nsis}{Number of predictors recruited by SIS. The default is n/log(n).}
}
\value{
the labels of first nsis largest active set of all predictors
}
\description{
To overcome challenges caused by ultra-high dimensionality,
Fan and Lv (2008) proposed a sure independence screening (SIS)
method, which aims to screen out the redundant features by
ranking their marginal Pearson correlations. The SIS method
is named after the SIS property, which states the selected subset
of features contains all the active ones with probability
approaching one.
}
\examples{

n <- 100
p <- 200
rho <- 0.5
data <- GendataLM(n, p, rho, error = "gaussian")
data <- cbind(data[[1]], data[[2]])
colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
data <- as.matrix(data)
X <- data[, 1:(ncol(data) - 1)]
Y <- data[, ncol(data)]
A <- SIS(X, Y, n / log(n))
A

}
\references{
Fan, J. and J. Lv (2008). Sure independence screening for ultrahigh dimensional feature space. Journal of the Royal Statistical Society: Series B (Statistical Methodology) 70(5),849–911.
}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DCSIS.R
\name{DCSIS}
\alias{DCSIS}
\title{Feature Screening via Distance Correlation Learning}
\usage{
DCSIS(X, Y, nsis = (dim(X)[1])/log(dim(X)[1]))
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}

\item{Y}{The response vector of dimension n * 1.}

\item{nsis}{Number of predictors recruited by DCSIS. The default is n/log(n).}
}
\value{
the labels of first nsis largest active set of all predictors
}
\description{
A sure independence screening procedure based on the distance correlation (DC-SIS).
The DC-SIS can be implemented as easily as the sure independence screening (SIS) procedure based on the Pearson correlation proposed by Fan and Lv(2008).
DC-SIS can be used directly to screen grouped predictor variables and multivariate response variables.
}
\examples{

n <- 100
p <- 200
rho <- 0.5
data <- GendataLM(n, p, rho, error = "gaussian")
data <- cbind(data[[1]], data[[2]])
colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
data <- as.matrix(data)
X <- data[, 1:(ncol(data) - 1)]
Y <- data[, ncol(data)]
A <- DCSIS(X, Y, n / log(n))
A

}
\references{
Fan, J. and J. Lv (2008). Sure independence screening for ultrahigh dimensional feature space. Journal of the Royal Statistical Society: Series B (Statistical Methodology) 70(5),849–911.

Li, R., W. Zhong, and L. Zhu (2012). Feature screening via distance correlation learning. Journal of the American Statistical Association 107(499), 1129–1139.
}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CAS.R
\name{CAS}
\alias{CAS}
\title{Category-Adaptive Variable Screening for Ultra-High Dimensional Heterogeneous Categorical Data}
\usage{
CAS(X, Y, nsis)
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}

\item{Y}{The response vector of dimension n * 1.}

\item{nsis}{Number of predictors recruited by CAS. The default is n/log(n).}
}
\value{
the labels of first nsis largest active set of all predictors
}
\description{
A category-adaptive screening procedure with high-dimensional heterogeneous data, which is to detect category-specific important covariates.
This proposal is a model-free approach without any specification of a regression model and an adaptive procedure
in the sense that the set of active variables is allowed to vary across different categories, thus making it more
flexible to accommodate heterogeneity.
}
\examples{

n <- 100
p <- 200
rho <- 0.5
data <- GendataLGM(n, p, rho)
data <- cbind(data[[1]], data[[2]])
colnames(data)[1:ncol(data)] <- c(paste0("X", 1:(ncol(data) - 1)), "Y")
data <- as.matrix(data)
X <- data[, 1:(ncol(data) - 1)]
Y <- data[, ncol(data)]
A <- CAS(X, Y, n / log(n))
A

}
\references{
Pan, R., Wang, H., and Li, R. (2016). Ultrahigh-dimensional multiclass linear discriminant analysis by pairwise sure independence screening. Journal of the American Statistical Association, 111(513):169–179.
}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PCSIS.R
\name{PCSIS}
\alias{PCSIS}
\title{Model-Free Feature Screening Based on the Projection Correlation}
\usage{
PCSIS(X, Y, nsis = (dim(X)[1])/log(dim(X)[1]))
}
\arguments{
\item{X}{The design matrix of dimensions n * p. Each row is an observation vector.}

\item{Y}{The response vector of dimension n * 1.}

\item{nsis}{Number of predictors recruited by PCSIS. The default is n/log(n).}
}
\value{
the labels of first nsis largest active set of all predictors
}
\description{
A model-free screening method is based on the projection correlation
which measures the dependence between two random vectors.
This projection correlation based method does not require specifying a
regression model, and applies to data in the presence of heavy tails
and multivariate responses. It enjoys both sure screening and
rank consistency properties under weak assumptions.
}
\examples{

# have_numpy=reticulate::py_module_available("numpy")
# if (have_numpy){
#  req_py()
library(MFSIS)
n=100;
p=200;
rho=0.5;
data=GendataLM(n,p,rho,error="gaussian")
data=cbind(data[[1]],data[[2]])
colnames(data)[1:ncol(data)]=c(paste0("X",1:(ncol(data)-1)),"Y")
data=as.matrix(data)
X=data[,1:(ncol(data)-1)];
Y=data[,ncol(data)];
# A=PCSIS(X,Y,n/log(n));A
# }else{
#   print('You should have the Python testing environment!')
#}

}
\references{
Zhu, L., K. Xu, R. Li, and W. Zhong (2017). Projection correlation between two random vectors. Biometrika 104(4), 829–843.

Liu, W., Y. Ke, J. Liu, and R. Li (2020). Model-free feature screening and FDR control with knockoff features. Journal of the American Statistical Association, 1–16.
}
\author{
Xuewei Cheng \email{xwcheng@hunnu.edu.cn}
}
